# -*- coding: utf-8 -*-
## Initialize PloneArticle product
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Initialize PloneArticle product
"""
__docformat__ = 'restructuredtext'

import os.path
import logging
LOG = logging.getLogger('PloneArticle')

__version__ = open(os.path.join(__path__[0], 'version.txt')).read().strip()
__version__ = __version__.lower()

# CMF imports
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore import DirectoryView
from Products.CMFCore.utils import ContentInit, ToolInit

# Archetypes imports
from Products.Archetypes.public import process_types, listTypes

# Products imports
from Products.PloneArticle import config
from Products.PloneArticle.pafti import PloneArticleFactoryTypeInformation
from Products.PloneArticle.pafti import manage_addPAFTIForm
from Products.PloneArticle.pafti import DynamicAllowedContentFTI
from Products.PloneArticle.pafti import manage_addDVTFTIForm

from Products.PloneArticle import content, proxy
from Products.PloneArticle import tool

from Products.PloneArticle import migration

# Register skin directories so they can be added to portal_skins
DirectoryView.registerDirectory('skins', config.GLOBALS)


def initialize(context):
    # Setup migrations
    migration.executeMigrations()
    migration.registerMigrations()

    # optional demo content
    if config.INSTALL_EXAMPLES:
        import examples

    # register the fti
    context.registerClass(
        PloneArticleFactoryTypeInformation,
        permission=ManagePortal,
        constructors=(manage_addPAFTIForm,),
        visibility=None)

    context.registerClass(
        DynamicAllowedContentFTI,
        permission=ManagePortal,
        constructors=(manage_addDVTFTIForm,),
        visibility=None)

    # initialize the content, including types and add permissions
    content_types, constructors, ftis = process_types(
        listTypes(config.PROJECTNAME),
        config.PROJECTNAME)

    permissions = {
        'PloneArticle': 'PloneArticle: Add PloneArticle',
        'PloneArticleMultiPage': "PloneArticle: Add PloneArticleMultiPage",
        }

    for atype, constructor, fti in zip(content_types, constructors, ftis):
        permission = permissions.get(atype.portal_type,
                                     'PloneArticle: Add PloneArticle')
        ContentInit(
            config.PROJECTNAME + ' Content',
            content_types=(atype,),
            permission= permission,
            extra_constructors=(constructor,),
            fti=(fti,),
        ).initialize(context)

    # Initialize the tool
    ToolInit(
        config.PROJECTNAME + ' Tool',
        tools=(tool.PloneArticleTool,),
        icon='tool.gif').initialize(context)



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: dp_articlefile.py 5793 2007-01-29 09:47:52Z roeder $
from funittest import DataProvider

class ArticleFile(DataProvider):
    pass
    
default = {'new_file_title':'An article',
           'new_file_description':'An article description',
           'new_file':'/Users/maik/Projects/funittest/README.txt'}
           
key="new_file_title"
articlefile = ArticleFile("PloneArticle", default, key)

provide = articlefile.provide

provide({'new_file_title':'File 1',
         'new_file_description':"File 1 Description"})



# -*- coding: utf-8 -*-
## Defines container of all proxy objects
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines container of all proxy objects
$Id: innercontentcontainer.py 6977 2007-12-28 11:45:13Z b_mathieu $
"""

__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo
from zope.interface import implements

# CMF imports
from Products.CMFCore import permissions as CCP
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.OrderedBaseFolder import OrderedContainer
from Products.Archetypes.public import BaseFolderMixin, registerType, listTypes
from Products.Archetypes.interfaces import IOrderedContainer

# Other imports
from Products.PloneArticle.interfaces import IInnerContentContainer, IBaseInnerContent
from Products.PloneArticle.pafti import DynamicAllowedContentFTI

class InnerContentContainer(OrderedContainer, BaseFolderMixin):
    """This folder is a container of inner content objects"""

    implements(IInnerContentContainer, IOrderedContainer)

    _at_fti_meta_type = DynamicAllowedContentFTI.meta_type

    meta_type = 'InnerContentContainer'
    portal_type = 'InnerContentContainer'
    global_allow = False
    filter_content_types = False
    allowed_content_types = ()
    security = ClassSecurityInfo()

    # from OrderedBaseFolder.OrderedBaseFolder:
    #
    # this ensure a proper call of manage_renameObject, since OrderedContainer
    # is the first parent class
    manage_renameObject = BaseFolderMixin.manage_renameObject
##     security.declareProtected(CCP.ModifyPortalContent, 'manage_renameObject')
##     def manage_renameObject(self, id, new_id, REQUEST=None):
##         """ rename the object """
##         ptool = getToolByName(self, 'plone_utils')
##         new_id = ptool.normalizeString(new_id)
##         objidx = self.getObjectPosition(id)
##         result = BaseFolderMixin.manage_renameObject(self, id, new_id, REQUEST)
##         self.moveObject(new_id, objidx)

##         return result

    security.declareProtected(CCP.AddPortalContent, 'invokeFactory')
    def invokeFactory(self, type_name, id, RESPONSE=None, *args, **kw):
        """Add new inner content"""

        # Check if content of specified portal type implements IBaseInnerContent
        atool = getToolByName(self, 'archetype_tool')
        types = [x for x in atool.listRegisteredTypes() if x['portal_type'] == type_name]
        if not types or types and not atool.typeImplementsInterfaces(types[0], (IBaseInnerContent,)):
            raise ValueError, 'Disallowed subobject type: %s' % type_name

        # Add inner content
        ttool = getToolByName(self, 'portal_types')
        args = (type_name, self, id, RESPONSE) + args
        new_id = ttool.constructContent(*args, **kw)
        if new_id is None or new_id == '':
            new_id = id
        return new_id

    security.declarePrivate('getCMFObjectsSubsetIds')
    def getCMFObjectsSubsetIds(self, objs):
        """Overriding OrderedContainer. See IOrderedContainer.
        This overrides a bug in OrderedContainer that prevents reordering
        with moveObjectsByDelta(...)"""

        ttool = getToolByName(self, 'portal_types')
        cmf_meta_types = ttool.listContentTypes(by_metatype=1)
        cmf_types = ttool.listContentTypes()
        arch_meta_types = [t['meta_type'] for t in listTypes()]
        types = cmf_types + arch_meta_types
        ids = [obj['id'] for obj in objs if obj['meta_type'] in types]
        return ids


registerType(InnerContentContainer)



# -*- coding: utf-8 -*-
## Defines LinkInnerContentField
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines LinkInnerContentProxy
"""

__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo
from zope.interface import implements

# CMF
from Products.CMFCore import permissions as CCP

# Archetypes imports
from Products.Archetypes.public import StringField, StringWidget, registerType,\
     Schema, ReferenceField, ComputedField, ComputedWidget

# Products imports
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import \
    ReferenceBrowserWidget

from Products.ATContentTypes.interface import IATLink, IATContentType

from Products.PloneArticle.proxy import BaseInnerContentProxy, \
    BaseInnerContentProxySchema
from Products.PloneArticle.interfaces import ILinkInnerContentProxy

# Defines schema
LinkInnerContentProxySchema = BaseInnerContentProxySchema.copy() + Schema((
    ComputedField(
        'remoteUrl',
        primary=True,
        expression="""context.getPrimaryValue('')""",
        widget=ComputedWidget(
            label='Remote url',
            label_msgid='label_remote_url',
            i18n_domain='plonearticle',
            ),
        ),
    ReferenceField(
        'referencedContent',
        relationship='article_link',
        widget=ReferenceBrowserWidget(
            label='Referenced link',
            label_msgid='label_referenced_link',
            i18n_domain='plonearticle',
            ),
        ),
    StringField(
        'attachedLink',
        widget=StringWidget(
            label='Link',
            label_msgid='label_link',
            i18n_domain='plonearticle',
            ),
        ),
    ))

class LinkInnerContentProxy(BaseInnerContentProxy):
    """Proxy implementing IATLink. It means this proxy has a remoteUrl
    method.

    remoteUrl returns attached link by default if existing otherwise returns
    the referenced content.
    """

    implements(ILinkInnerContentProxy)
    security = ClassSecurityInfo()

    schema = LinkInnerContentProxySchema

    # You can only reference content implementing IATLink interface
    referenceable_interfaces = (IATContentType,)

    security.declareProtected(CCP.View, 'getPrimaryValue')
    def getPrimaryValue(self, default=None):
        """getPrimaryValue proxy method for links
        Returns referenced absolute url or attached link.
        Attached link has priority on referenced url.
        """

        # If an attached link is defined return its value
        field = self.getField('attachedLink')
        if field.get_size(self) > 0:
            accessor = field.getAccessor(self)
            return accessor()

        # Attached field is empty. Use referenced content
        referenced_content = self.getReferencedContent()
        if referenced_content is not None:
            return referenced_content.absolute_url()

        return default

    security.declareProtected(CCP.View, 'SearchableText')
    def SearchableText(self):
        """indexing links is useless
        """
        return self.Title() + self.Description()

    security.declareProtected(CCP.View, 'getLinkIcon')
    def getLinkIcon(self):
        """return link_icon or referenced conten icon
        """
        attachField = self.getField('attachedLink')
        if attachField.get_size(self) > 0 :
            return 'link_icon.gif'
        elif self.getReferencedContent() is not None :
            return self.getReferencedContent().getIcon(True)    
        return self.getIcon(True)    



registerType(LinkInnerContentProxy)



## Script (Python) "pa_innerimage_upload"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=field_name=None, new_file_title='', new_file_description='', new_file=None, widget_index=None
##title=Upload image
##

return context.pa_innercontent_upload(
    field_name=field_name,
    widget_index=widget_index, 
    max_size_property="imageMaxSize", 
    max_size_field_name="attachedImage",
    title=new_file_title, 
    description=new_file_description, 
    attachedImage=new_file)



# -*- coding: utf-8 -*-
## Test case for PloneArticle content type
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test case for PloneArticle content type
"""

__docformat__ = 'restructuredtext'

#
from Acquisition import Implicit

# Products imports
from Products.PloneArticle.tests.common import BaseTestCase

class DummyAcquired(Implicit):
    """
    """
    portal_type = 'DummyAcquired'


class PloneArticleTestCase(BaseTestCase):
    """
    This class contains all test cases for PloneArticle content type.
    This content type is defined in : Products.PloneArticle.content.article
    """

    def afterSetUp(self):
        """ """
        self.loginAsPortalOwner()
        self.article = self.addPloneArticle(self.portal, 'article')
        self.logout()
    
    def testSkeleton(self):
        """Add new PloneArticle content and test its class skeleton"""        
        article = self.article
        
        # Test methods
        self.failUnless(hasattr(article, 'Title'))
        self.failUnless(hasattr(article, 'setTitle'))
        self.failUnless(hasattr(article, 'Description'))
        self.failUnless(hasattr(article, 'setDescription'))
        self.failUnless(hasattr(article, 'getText'))
        self.failUnless(hasattr(article, 'setText'))
        self.failUnless(hasattr(article, 'setImages'))
        self.failUnless(hasattr(article, 'getImages'))
        self.failUnless(hasattr(article, 'setFiles'))
        self.failUnless(hasattr(article, 'getFiles'))
        self.failUnless(hasattr(article, 'setLinks'))
        self.failUnless(hasattr(article, 'getLinks'))

    def testInterfaces(self):
        """
        Test presence of interfaces (+z3 interfaces obtained by ZCML)
        """
        from Products.PloneArticle.interfaces import INonStructuralFolder
        from Products.PloneArticle.interfaces import IPloneArticle
        from Products.ATContentTypes.interface import IATDocument

        article = self.article

        self.failUnless(INonStructuralFolder.isImplementedBy(article))
        self.failUnless(IPloneArticle.isImplementedBy(article))
        self.failUnless(IATDocument.isImplementedBy(article))

    def testVersionAtCreation(self):
        from Products.PloneArticle.version import CURRENT_ARTICLE_VERSION
        article = self.article

        self.assertEquals(article.getPAVersion(), CURRENT_ARTICLE_VERSION) 

    def testGetArticleObject(self):
        article = self.article

        dummy = DummyAcquired()
        acquired = dummy.__of__(article)
        result = acquired.getArticleObject()
        self.assertEqual(result, article)

    def testCopyArticle(self):
        self.loginAsPortalOwner()
        article = self.article
        
        # Add image
        image = self.addContent("Image", self.portal, "image1")
        
        # Reference this image to article
        data = [{
            'id': 'image1',
            'title': ('Inner image', {}), 
            'referencedContent': (image.UID(), {})}]
        article.setImages(data)
        
        # Copy article to new folder
        folder = self.addContent("Folder", self.portal, "folder")
        cb = self.portal.manage_copyObjects(ids=[article.getId()])
        folder.manage_pasteObjects(cb_copy_data=cb)
        article_copy = getattr(folder, article.getId())
        
        # Get references of proxy image
        refs = [(x.targetUID, x.relationship)
                for x in article.images.image1.getReferenceImpl()]
        refs_copy = [(x.targetUID, x.relationship)
                     for x in article_copy.images.image1.getReferenceImpl()]
        self.assertEqual(refs, refs_copy)

    def test_canSetDefaultPage(self):
        article = self.article
        self.loginAsPortalOwner()
        self.failIf(article.canSetDefaultPage())
        
    def test_Description(self):
        article = self.article
        text = """This is a dummy text"""
        self.loginAsPortalOwner()
        field = article.getField('description')
        mutator = field.getMutator(self.article)
        mutator(text)
        accessor = field.getEditAccessor(self.article)
        self.assertEquals(accessor(), text)
        
    def test_SearchableText(self):
        """Test if article SearchableText method returns the correct value 
        for SearchableText index"""
        
        article = self.article
        self.loginAsPortalOwner()
        article.setTitle('Article title')
        file_values = ({'id': 'object_1',
                        'title': 'Title 1',
                        'attachedFile': 'This is a dummy text'},
                       {'id': 'object_2',
                        'title': 'Title 2',
                        },)
        article.setFiles(file_values)
        expected_value = '1 2 a article article dummy is text this title title title'
        value = article.SearchableText()
        value_list = [x.lower() for x in value.split(" ") if x]
        value_list.sort()
        value = " ".join(value_list)
        self.assertEquals(value, expected_value)
    
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(PloneArticleTestCase))
    return suite



# -*- coding: utf-8 -*-
## Defines BaseInnerContentWidget
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines BaseInnerContentWidget
"""

__docformat__ = 'restructuredtext'

from time import strftime

# Zope imports
from AccessControl import ClassSecurityInfo
from OFS.Image import File

# Archetypes imports
from Products.Archetypes.Registry import registerWidget
from Products.Archetypes.Widget import TypesWidget

_marker = []

class BaseInnerContentWidget(TypesWidget):
    """"""

    _properties = TypesWidget._properties.copy()
    _properties.update({
        'macro' : "pa_baseinnercontentwidget",
        'new_content_macro': "pa_baseinnercontentwidget",
        })

    security = ClassSecurityInfo()

    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False):
        """
        Calls process_form on all sub fields.

        Like all process_form methods, returns a tuple and a dictionary
        inside a tuple.

        The tuple contains all results of calling process_form on sub fields
        of inner content:

        ({'description': ('', {}),
          'id': ('imageinnercontentproxy.2007-01-30.8023847640', {}),
          'title': ('', {})})
        """

        # Get form keys and just keep those related to the field
        # Key : <field id>_<sub field id>_<sub field index>
        form_keys = form.keys()
        # field_name can be for example "images"
        field_name = field.getName()
        # key_prefix can be for example "images_innercontent"
        key_prefix = '%s_innercontent' % field_name
        # position_key can be for example "images_innercontent_position"
        position_key = '%s_position' % key_prefix
        key_prefix_length = len(key_prefix)
        field_keys = [k for k in form_keys if k.startswith(key_prefix) and k not in (position_key,)]

        # The filtered field_keys can be for example:
        # ['images_innercontent160231_tempFileIndex',
        #  'images_innercontent160231_isTemporary',
        #  'images_innercontent160231_id',
        #  'images_innercontent160231_description',
        #  'images_innercontent160231_title']

        # Group all results in form by inner content widget.
        sub_forms = {}
        for field_key in field_keys:
            # Get sub field index
            sub_field_index = field_key[key_prefix_length:(key_prefix_length+6)]
            sub_field_name = field_key[(key_prefix_length+7):]
            if not sub_forms.has_key(sub_field_index):
                sub_forms[sub_field_index] = {}

            sub_forms[sub_field_index][sub_field_name] = form.get(field_key)

        # The resulting subforms dictionary may look like this:
        # {'160231': {'tempFileIndex': 0,
        #             'description': '',
        #             'id': 'imageinnercontentproxy.2007-01-30.3505314205',
        #             'isTemporary': 1,
        #             'title': ''}}

        # Sort sub forms py position
        inner_content_position = form.get(position_key, [])

        # The inner_content_position may look like this:
        # ['160231']

        sub_forms = [sub_forms[k] for k in inner_content_position]

        # The sub_forms may look like this now:
        # [{'tempFileIndex': 0,
        #   'description': '',
        #   'id': 'imageinnercontentproxy.2007-01-30.3505314205',
        #   'isTemporary': 1,
        #   'title': ''}]

        inner_content_schema = field.getInnerContentSchema(instance)
        inner_content_fields = inner_content_schema.fields()
        # inner_content_fields may look like this now:
        # [<Field id(string:rw)>,
        #  <Field title(string:rw)>,
        #  <Field description(text:rw)>,
        #  <Field referencedContent(reference:rw)>,
        #  <Field image(computed:r)>,
        #  <Field attachedImage(image:rw)>]

        value = []

        request = instance.REQUEST

        for sub_form in sub_forms:
            inner_content_values = {}
            for inner_content_field in inner_content_fields:
                widget = inner_content_field.widget
                inner_content_value = widget.process_form(instance, inner_content_field, sub_form, empty_marker=_marker)
                if inner_content_value is _marker or inner_content_value is None:
                    continue
                inner_content_values[inner_content_field.getName()] = inner_content_value
            value.append(inner_content_values)

        return tuple(value), {}

    security.declarePublic('makeWidgetId')
    def makeWidgetId(self, fieldName, widget_index):
        """
        """
        return '%s-innercontent%06d' % (fieldName, widget_index)

    security.declarePublic('makeInnerContentWidgetId')
    def makeInnerContentWidgetId(self, fieldName, index, icf_name):
        """
        Return an id for a proxy field name

        @param fieldName: InnerContentField name
        @param index: proxy content index
        @param icf_name: proxy field name
        """
        return '%s_innercontent%06d_%s' % (fieldName, index, icf_name)

    security.declarePublic('innerContentWidget')
    def innerContentWidget(self, field_name, index, inner_content,
                           inner_content_field, mode='edit'):
        """
        """
        icf_name = inner_content_field.getName()
        widget_field_name = self.makeInnerContentWidgetId(field_name, index,
                                                          icf_name)

        return inner_content.widget(widget_field_name,
                                    field=inner_content_field,
                                    mode=mode)

    security.declarePublic('generateWidgetIndex')
    def generateWidgetIndex(self):
        """
        """
        # we use hour-minutes-seconds as a unique number
        # should be valid for an editing round within the same form!
        return int(strftime("%H%M%S"))

registerWidget(BaseInnerContentWidget,
               title='BaseInnerContent',
               description=(''),
               used_for=('Products.PloneArticle.field.BaseInnerContentWidget',)
               )



# -*- coding: utf-8 -*-
## Tools to use in test cases
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Tools to use in test cases
"""

__docformat__ = 'restructuredtext'

# Python imports
import os

def openTestFile(name):
    dirname = os.path.dirname(__file__)
    path = os.path.join(dirname, 'input', name)
    fd = open(path, 'rb')
    return fd

def loadFile(name, size=0):
    """Load file from testing directory
    """
    fd = openTestFile(name)
    data = fd.read()
    fd.close()
    return data



# -*- coding: utf-8 -*-
## PloneArticle
## 
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#
# from CMFPlone 2.5
#
# CMFPlone/LICENCE.txt:
#   The Plone Content Management System is built on the Content 
#   Management Framework (CMF) and the Zope Application Server.
#   Plone is copyright 2000-2006 Plone Foundation et al.
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
# Zope 3.1-style messagefactory module for Zope <= 2.9 (Zope 3.1)
#

# BBB: Zope 2.8 / Zope X3.0

from zope.i18nmessageid import MessageIDFactory
msg_factory = MessageIDFactory('plonearticle')

def ArticleMessageFactory(ustr, default=None, mapping=None):
    message = msg_factory(ustr, default)
    if mapping is not None:
        message.mapping.update(mapping)
    return message

model_msg_factory = MessageIDFactory('plonearticle-model')

def ModelMessageFactory(ustr, default=None, mapping=None):
    message = model_msg_factory(ustr, default)
    if mapping is not None:
        message.mapping.update(mapping)
    return message



# Copyright (C) 2003-2006 Ingeniweb SAS

# This software is subject to the provisions of the GNU General Public
# License, Version 2.0 (GPL).  A copy of the GPL should accompany this
# distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY,
# AGAINST INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE

# More details in the ``LICENSE`` file included in this package.

"""
    Plone Article migration module
"""

# Python imports
from StringIO import StringIO
from types import IntType
import sys
import itertools

import transaction

# Zope imports
from OFS.Image import File
from zExceptions import NotFound

# We need to run some methods with all permissions
from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.User import UnrestrictedUser

# CMF imports
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.interfaces.base import IBaseUnit
from Products.CMFPlone.utils import safe_hasattr

from Products.PloneArticle.version import getArticleVersionFor
from Products.PloneArticle.migration import applyArticles
from Products.PloneArticle import LOG

def generateUniqueId(type_name=None):
    """
        Generate an id for the content
        This is not the archetype's uid.
    """
    from DateTime import DateTime
    from random import random

    now = DateTime()
    time = '%s.%s' % (now.strftime('%Y-%m-%d'), str(now.millis())[7:])
    rand = str(random())[2:6]
    prefix = ''
    suffix = ''

    if type_name is not None:
        prefix = type_name.replace(' ', '_') + '.'
    prefix = prefix.lower()

    return prefix + time + rand + suffix
def get322Contents(article, orderedListName, internal=True):
    """
        Return all the images that are linked in an article.

        article is a 3.x.x article
        orderedListName is the name of the attributes which contain the ordered
        list of contents.
             It can be either '__ordered_attachment_refs__', '__ordered_image_refs__'
             or '__ordered_link_refs__'.
        if internal is True, return Images detected as internal, else
        return images given in reference.
    """

    LOG.debug("migration.get322Contents parameters:\n %s %s %s" % (article.absolute_url(), orderedListName, repr(internal)))
    if not safe_hasattr(article, "at_references"):
        raise ValueError("Article has no at_references attribute.")

    result = []
    if internal:
        # During migration from old Plone and Zope versions some references
        # are inconsistant.
        # To be sure to migrate all inner content we **guess** that only one inner
        # type contains word in portal_type or in meta_type to filter out other
        # inner types.
        word = ''
        if orderedListName == '__ordered_attachment_refs__':
            word = 'file'
        elif orderedListName == '__ordered_image_refs__':
            word = 'image'
        elif orderedListName == '__ordered_link_refs__':
            word = 'link'
        else:
            # XXX Ouch, it's some unknown orderedListName: try and pray.
            word = 'file'

        result.extend([obj for obj in article.objectValues()
                       if word in obj.portal_type.lower() or word in obj.meta_type.lower()])
        LOG.debug("inner content for word %s:\n %s" % (word, repr(result)))

    try:
        oList = getattr(article, orderedListName)
    except AttributeError, e:
        # There's something bad there. As we don't want the migration fail here
        # we put a log in 'error' level.
        LOG.error("migration.get322Contents:\n%s attribute does'nt exist for %s" % (orderedListName, article.absolute_url()))
        return result

    for refUid in oList.getItems():
        ref = getattr(article.at_references, refUid)
        ## don't use hasattr (it uses aquisition) use safe_hasattr
        try:
            obj = ref.getTargetObject()
        except AttributeError, e: ## in some case, there is some ghost references...
            LOG.debug("Atributerror during migration on %s" % e)
            continue ## just ignore it...
        if safe_hasattr(article, obj.id) and internal:
            if obj not in result:
                result.append()
        elif not safe_hasattr(article, ref.getTargetObject().id) and not internal:
            result.append(ref.getTargetObject())

    return result


def migrateImages322to400(article, out):
    ## looking for old images
    intImages = get322Contents(article, "__ordered_image_refs__", internal=True)
    extImages = get322Contents(article, "__ordered_image_refs__", internal=False)

    ## article
    ##     images (ImageInnerContent)
    ##         ImageInnerContentProxies (as many as images)
    ##             attachedImage or
    ##             referencedContent
    ##             (image) -> just a computed field
    ##             title
    ##             description
    ##
    ##     links
    ##     files

    values = []
    for image in intImages:
        value = { ## this are the fields of ImageInnerContentProxy
            "attachedImage": (image.getImage(), {}),
            "title": (image.Title(), {}),
            "description": (image.Description(), {}),
            "id": (generateUniqueId("imageProxy"), {}),
        }
        values.append(value)
    for image in extImages:
        value = { ## this are the fields of ImageInnerContentProxy
            "referencedContent": (image, {}),
            "title": (image.Title(), {}),
            "description": (image.Description(), {}),
            "id": (generateUniqueId("imageProxy"), {}),
        }
        values.append(value)

    # XXX Something make us loose right, but we are pragmatic
    current_user = getSecurityManager().getUser()
    newSecurityManager(None, UnrestrictedUser('manager', '', ['Manager'], []))
    article.setImages(values)
    newSecurityManager(None, current_user)

def migrateFiles322to400(article, out):
    intFiles = get322Contents(article, "__ordered_attachment_refs__", internal=True)
    extFiles = get322Contents(article, "__ordered_attachment_refs__", internal=False)

    values = []
    for file in intFiles:
        value = { ## this are the fields of ImageInnerContentProxy
            "attachedFile": (file.getFile(), {}),
            "title": (file.Title(), {}),
            "description": (file.Description(), {}),
            "id": (generateUniqueId("fileProxy"), {}),
        }
        values.append(value)
    for file in extFiles:
        value = { ## this are the fields of ImageInnerContentProxy
            "referencedContent": (file, {}),
            "title": (file.Title(), {}),
            "description": (file.Description(), {}),
            "id": (generateUniqueId("fileProxy"), {}),
        }
        values.append(value)

    # XXX Something make us loose right, but we are pragmatic
    current_user = getSecurityManager().getUser()
    newSecurityManager(None, UnrestrictedUser('manager', '', ['Manager'], []))
    article.setFiles(values)
    newSecurityManager(None, current_user)

def migrateLinks322to400(article, out):
    intLinks = get322Contents(article, "__ordered_link_refs__", internal=True)
    extLinks = get322Contents(article, "__ordered_link_refs__", internal=False)

    values = []
    for link in intLinks:
        value = { ## this are the fields of ImageInnerContentProxy
            "attachedLink": (link.getRemoteUrl(), {}),
            "title": (link.Title(), {}),
            "description": (link.Description(), {}),
            "id": (generateUniqueId("linkProxy"), {}),
        }
        values.append(value)
    for link in extLinks:
        value = { ## this are the fields of ImageInnerContentProxy
            "referencedContent": (link, {}),
            "title": (link.Title(), {}),
            "description": (link.Description(), {}),
            "id": (generateUniqueId("linkProxy"), {}),
        }
        values.append(value)

    # XXX Something make us loose right, but we are pragmatic
    current_user = getSecurityManager().getUser()
    newSecurityManager(None, UnrestrictedUser('manager', '', ['Manager'], []))
    article.setLinks(values)
    newSecurityManager(None, current_user)


def migrateModel322to400(article, out):
    out.write("Models migration: ")
    if article.model in  ('pa_model', 'plonearticle_model'):
        newModel = "pa_model1"
    elif article.model.startswith('plonearticle_model'):
        ## We keep only the last number as known models are between 1 and 6
        newModel = 'pa_model' + article.model[-1]
    else:
        newModel = article.model
    article.setLayout(newModel)
    out.write("old model %s || new model %s\r\n" % (article.model, repr(article.getLayout())))

def clean322to400(article, out):

    ## Do not remove ids of objects like files or images, since
    ## when deleted, they do not apear anymore even if they are stored in
    ## the proxy.

    attributesToRemove = [
        "__ordered_attachment_refs__",
        "__ordered_image_refs__",
        "__ordered_link_refs__",
        "_locked_by",
        "_locked_date",
        "_objects",
        "_unlocked_by",
        "_unlocked_date",
        "model",
    ]
    for attr in attributesToRemove:
        try:
            delattr(article, attr)
        except AttributeError:
            pass


def migrateArticle322to400(article, out):

    out.write("Migrating article at '%s'\r\n" % article.absolute_url())

    migrateImages322to400(article, out)
    migrateFiles322to400(article, out)
    migrateLinks322to400(article, out)

    migrateModel322to400(article, out)

    clean322to400(article, out)
    out.write("success!\r\n\r\n")

def migrateArticle(article, out):
    migrateArticle322to400(article, out)

def migrateArticles(portal, article_brains, out):
    """
        Migrate all PloneArticle 3.x to PloneArticle 4.0.0
        article_brains is a collection of brains of article
    """

    plonearticle_tool = getToolByName(portal, 'plonearticle_tool')

    out.write('Migrate PloneArticle v3.x to v4.0.0.\r\n')

    nb_articles = 0
    already = 0

    out.write("There are %d PloneArticle objects.\r\n\r\n" % len(article_brains))

    # Get all PloneArticle in v3.x
    for article_brain in article_brains:
        try:
            article = article_brain.getObject()

            # Check if it has not been done before
            if article.getPAVersion() ==  (4, 0, '0-beta4'):
                already += 1
                continue

            # This is an article from v3.x
            parent = article.getParentNode()
            nb_articles += 1
            migrateArticle(article, out)
        except NotFound, e:
            out.write("%s not found.\r\n" % e)

    left = len(article_brains) - already - nb_articles
    out.write('%s article(s) migrated.\r\n' % nb_articles)
    out.write('%s article(s) were already migrated.\r\n' % already)
    out.write('%s article(s) left unmigrated.\r\n' % left)
    if left > 0:
        out.write('Please launch migration process again.\r\n')
    else:
        out.write('\n')
    out.write('Migration from PloneArticle v3.x to v4.0.0 finished.\r\n')

def v3_v4beta1(portal):
    """Migrate from v3.x to v4.0.0beta1    """

    out = StringIO()
    portal_catalog = getToolByName(portal, 'portal_catalog')
    article_brains = portal_catalog(portal_type='PloneArticle')
    migrateArticles(portal, article_brains, out)
    return (out.getvalue(),)

###############################################################################

def beta3_beta4(portal):
    """4.0.0beta3 to beta4"""
    out = []

    version = getArticleVersionFor('4.0.0-beta4')

    def modifyArticle(article):
        # fix version. Earlier v4 articles don't have one
        # post beta4 should have an integer number (value >= 0)
        if type(article.getPAVersion()) is not IntType:
            article.setPAVersion(version)

        # rename proxy if necessary
        file_proxies = tuple(article.getFiles()) + tuple(article.getImages())
        for proxy in file_proxies:
            proxy.renameFromFileName()

    applyArticles(portal, modifyArticle)

    out.append("Set version information on older articles")
    out.append("Renamed proxy files and images ids to stored filename")
    return out


###############################################################################

def beta5_beta6(portal):
    """4.0.0beta5 to beta6"""
    out = []

    def modifyModel(article):
        # fix model. Earlier beta6 articles model migration is broken
        model = article.layout
        if model is None:
            article.setLayout('pa_model1')
        elif model.startswith('pa_odel'):
            article.setLayout('pa_model' + model[-1])

    applyArticles(portal, modifyModel)
    out.append("Fix model migration on older articles")

    if 'portal_article' in portal.objectIds():
        portal.manage_delObjects(['portal_article',])
        out.append("Delete old article tool")

    return out



# -*- coding: utf-8 -*-
## Defines FileInnerContentField
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines FileInnerContentField
"""

__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo
from zope.interface import implements

# Archetypes imports
from Products.Archetypes.Registry import registerField

# Products imports
from Products.PloneArticle.field import BaseInnerContentField
from Products.PloneArticle.interfaces import IFileInnerContentField

class FileInnerContentField(BaseInnerContentField):
    """Defines proxy managing files"""
 
    implements(IFileInnerContentField)
    _properties = BaseInnerContentField._properties.copy()
    _properties.update({
        'inner_portal_type' : 'FileInnerContentProxy',
        })

    security = ClassSecurityInfo()

registerField(FileInnerContentField,
              title='FileInnerContent',
              description=('Used to store FileInnerContentProxy objects.')
             )



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: test_delete_file.py 5783 2007-01-21 17:59:47Z roeder $
import unittest
from funittest import logical
from funittest import dataprovider
from funittest import precondition
from funittest import interpreter
from funittest import TestCase

import os

class TestDeleteFile(TestCase):

    def setUp(self):    
        TestCase.setUp(self)
        interpreter.open("")        
        user=dataprovider.cmfplone.user.get('sampleadmin')
        logical.cmfplone.application.change_user(user)        
        self.article = dataprovider.plonearticle.article.get("Article 1")
        files = [dataprovider.plonearticle.articlefile.get("File 1"),]
        images = []
        links = []
        precondition.plonearticle.article.existing_article_with_attachments(self.article, files, images, links)

    def test_delete_file(self):
        "Delete a file in an article"
        logical.plonearticle.article.delete_file(file)
        logical.plonearticle.article.save_article(self.article)
        
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
## Defines LinkInnerContentWidget
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines LinkInnerContentWidget
"""

__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo

# Archetypes imports
from Products.Archetypes.Registry import registerWidget

# Products imports
from Products.PloneArticle.widget import BaseInnerContentWidget

class LinkInnerContentWidget(BaseInnerContentWidget):
    """"""
    
    _properties = BaseInnerContentWidget._properties.copy()
    _properties.update({
        'macro' : "pa_linkinnercontentwidget",
        })

    security = ClassSecurityInfo()
    
registerWidget(LinkInnerContentWidget,
               title='LinkInnerContent',
               description=(''),
               used_for=('Products.PloneArticle.field.linkinnercontent.LinkInnerContentField',)
               )



# -*- coding: utf-8 -*-
## PloneArticle
##
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
from Products import PloneArticle as PRODUCT
import os.path

version = PRODUCT.__version__
modname = PRODUCT.__name__

# (major, minor, revision)
# i.e 4.0.0-beta4 CVS/SVN (UNRELEASED) => (4, 0, 0, 'beta2')
ver_tup = version.split(' ')
vers = ver_tup[0]
major, minor, suffix =  vers.split('.')
suffix = suffix.split('-')
bugfix = suffix[0]
release = len(suffix) > 1 and suffix[1] or ''
numversion = (int(major), int(minor), int(bugfix), release)

license     = 'GPL'
copyright   = '''(c) 2006 Ingeniweb'''

author      = 'Ingeniweb <support@ingeniweb.com>'
author_email= 'support@ingeniweb.com'

short_desc  = 'Article content type for Plone'
long_desc   = '''
'''

copyright_text = '''
'''

web         = 'http://plone.org/products/plonearticle'
ftp         = ''
mailing_list= ''
bugtracker  = 'http://sourceforge.net/tracker/?atid=541550&group_id=74634'



# Copyright (c) 2003-2006 Ingeniweb SAS

# This software is subject to the provisions of the GNU General Public
# License, Version 2.0 (GPL).  A copy of the GPL should accompany this
# distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY,
# AGAINST INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE

# More details in the ``LICENSE`` file included in this package.

"""
Test case for PloneArticle content type
"""

__docformat__ = 'restructuredtext'

import os
from Acquisition import Implicit
from Globals import package_home

# Products imports
from Products.PloneArticle.config import GLOBALS
from Products.PloneArticle.tests.common import BaseTestCase


class TestMigration(BaseTestCase):

    def importArticleV3(self, container):
        """
            Import article V3 from zexp
        """
        
        article_id = 'articlev3'
        zexp_path = os.path.join(package_home(GLOBALS), 'tests', '%s.zexp' % article_id)
        container._importObjectFromFile(zexp_path, verify = 1, set_owner = 1)
        return getattr(container, article_id)

    def testPloneArticleMigrationV3ToV4(self):
        self.loginAsPortalOwner()
        
        # Import article in v2.x
        article = self.importArticleV3(self.portal)
        article.reindexObject()
        article_id = article.getId()
        
        # Attributes before migration
        ### XXX for tests, this value have to be really hardcoded
        article_id = article.getId()
        article_title = article.Title()
        article_description = article.Description()
        article_text = article.getText()
        
        self.failUnless(hasattr(article, '__ordered_attachment_refs__'))
        self.failUnless(hasattr(article, '__ordered_image_refs__'))
        self.failUnless(hasattr(article, '__ordered_link_refs__'))
        
        
        # Migrate to 4
        Migrator().migrate(self.portal)
        
        # Attributes after migration
        ### XXX check that all is allright.
        self.logout()

class TestMigrationPath(BaseTestCase):
    """The path to the actual version should exist"""

    def testMigrationPath(self):

        from Products.CMFCore.utils import getToolByName
        from Products.PloneArticle.config import PLONEARTICLE_TOOL
        from Products.PloneArticle.tool import _upgradePaths

        pa_tool = getToolByName(self.portal, PLONEARTICLE_TOOL)
        version_info = pa_tool.getVersionFromFS()
        new_versions = [x[0] for x in _upgradePaths.values()]
        self.failUnless(
            version_info[1] in new_versions,
            "No migrator found for version '%s'. Please correct migration/__init__.py\n"
            "You might consider this as a warning if you're not testing a released version"
            % version_info[1])
        return

#FIXME: this is just a non-functionnal skeleton of a test, don't expect it to
#pass. For now we will just return an empty test suite. Too bad.
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # Uncomment below line to perform the migration test
    #suite.addTest(makeSuite(TestMigration))
    suite.addTest(makeSuite(TestMigrationPath))
    return suite



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: dp_articlelink.py 5766 2007-01-15 16:52:32Z roeder $
from funittest import DataProvider

class ArticleLink(DataProvider):
    pass
    
default = {'new_title':'An article link',
           'new_description':'An article link description',
           'new_url':'http://www.plone.org',}

key="new_title"
articlelink = ArticleLink("PloneArticle", default, key)

provide = articlelink.provide

provide({'new_title':'Link 1',
         'new_description':"Link 1 Description",
         'new_url':'http://www.plone.org'})

        
        


# -*- coding: utf-8 -*-
## PloneArticle
## 
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test case for PloneArticle FTIs
"""

__docformat__ = 'restructuredtext'

from Products.Archetypes.atapi import registerType

# Products imports
from Products.PloneArticle.pafti import PloneArticleFactoryTypeInformation
from Products.PloneArticle.tests.common import BaseTestCase

class PAFTITestCase(BaseTestCase):
    """
    Tests for pafti.PloneArticleFactoryTypeInformation
    """
    def afterSetUp(self):

        # which one is best for our tests?
        #self.fti = PloneArticleFactoryTypeInformation('PloneArticle').__of__(self.portal)
        self.fti = self.portal.portal_types['PloneArticle']

    def test_PA_content_FTI(self):
        self.assert_(isinstance(self.fti, PloneArticleFactoryTypeInformation))
        
    def test_getAvailableReferenceableAttachmentTypes(self):
        self.assertEquals(self.fti.getAvailableReferenceableAttachmentTypes(),
                          ['File'])

    def test_getAvailableReferenceableImageTypes(self):
        value = self.fti.getAvailableReferenceableImageTypes()
        value.sort()
        expected_value = ['Image', 'News Item',]
        expected_value.sort()
        self.assertEquals(value, expected_value)

    def test_getAvailableReferenceableLinkTypes(self):
        value = self.fti.getAvailableReferenceableLinkTypes()
        value.sort()
        expected_value = ['Image', 'Topic', 'Large Plone Folder', 'Document',
            'PloneArticleMultiPage', 'Favorite', 'Event', 'Folder', 'Link', 
            'News Item', 'File', 'PloneArticle',]
        expected_value.sort()
        self.assertEquals(value, expected_value)

class DynamicAllowedContentFTITestCase(BaseTestCase):
    """
    Tests for pafti.DynamicAllowedContentFTI
    """

    def afterSetUp(self):
        self.fti = self.portal.portal_types['InnerContentContainer']

    def test_allowType(self):
        """Test that only IBaseInnerContent types are allowed"""
        fti = self.fti
        self.assertEquals(fti.allowType('FileInnerContentProxy'), True)
        self.assertEquals(fti.allowType('ImageInnerContentProxy'), True)
        self.assertEquals(fti.allowType('File'), False)
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(PAFTITestCase))
    suite.addTest(makeSuite(DynamicAllowedContentFTITestCase))
    return suite




## Script (Python) "pa_innercontent_upload"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=field_name, widget_index=None, max_size_property=None, max_size_field_name=None, **kwargs
##title=
##

from Products.CMFCore.utils import getToolByName

request = context.REQUEST
pa_tool = getToolByName(context, 'plonearticle_tool')
utils = getToolByName(context, 'plone_utils')
field = context.getField(field_name)
template = context.restrictedTraverse(path='pa_temporary_innerimage_widget')

# help utranslate to write with proper encoding
charset = utils.getSiteEncoding()
request.RESPONSE.setHeader('Content-Type', 'text/html; charset=%s' % charset)

# Get size and check against max allowed size
if max_size_property is not None:
    # Check max size property
    file_to_check = kwargs.get('max_size_field_name', None)
    
    if file_to_check is not None:
        file_to_check.seek(0, 2)
        size = file_to_check.tell()
        file_to_check.seek(0)
        max_size = context.getTypeInfo().getProperty(max_size_property, None)
    
        if max_size is not None and size > max_size:
            message = context.utranslate(
                u'max_file_size_exceeded_error',
                mapping={
                    "max_size": context.getObjSize(context, max_size),
                    },
                default=u"Error: maximum allowed file size is ${max_size}",
                domain="plonearticle")
            return template(from_upload=True, skip_widget=True, fail_reason=message)

# Upload new content
accessor = field.getAccessor(context)
existing_ids = [c.getId() for c in accessor()]
tmp_content = field.getTemporaryInnerContent(context)
mutator = field.getMutator(context)
try:
    value = {"id": tmp_content.getId(),}
    value.update(kwargs)
    mutator((value,), update=True)
except:
    exception = context.plone_utils.exceptionString()
    message = "Uploaded content error: %s" % exception
    return template(from_upload=True, skip_widget=True, fail_reason=message)


# FIXME: This use case cannot happen. Maybe delete it
# Check if content has been added to article
new_inner_contents = [c for c in accessor()
                     if c.getId() not in existing_ids]

if len(new_inner_contents)!=1:
    exception = "Uploaded content has not been added"
    message = "Uploaded content error: %s" % exception
    return template(from_upload=True, skip_widget=True, fail_reason=message)

inner_content = new_inner_contents[0]

# Render template
widget = field.widget
if widget_index is None:
    widget_index = widget.generateWidgetIndex()   
widget_index = int(widget_index)

return template(from_upload=True,
                inner_content=inner_content,
                field=field,
                field_name=field_name,
                widget=widget,
                widget_index=widget_index)


# -*- coding: utf-8 -*-
## Test case for all InnerContentProxy classes
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test case for all InnerContentProxy classes
"""

__docformat__ = 'restructuredtext'

# Python imports
from types import TupleType

# Zope imports
from zope.interface import Interface

# CMF imports
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.tests.utils import mkDummyInContext
from Products.Archetypes.tests.test_fields import Dummy
from Products.Archetypes.public import Schema, FileField, ImageField, \
    StringField, BaseContent

# Products imports
from Products.PloneArticle.tests.common import BaseTestCase, openTestFile, \
     loadFile
from Products.PloneArticle import proxy as pa_proxies
from Products.PloneArticle.field import InnerContentContainer

from Products.CMFPlone.utils import _createObjectByType

test_types_names = [
    'BaseInnerContentProxy',
    'FileInnerContentProxy',
    'ImageInnerContentProxy',
    'LinkInnerContentProxy',
    ]

schema = Schema((
   FileField('file'),
   ImageField('image'),
   StringField('remoteUrl'),
   ),)

test_proxies = (
    ('file', 'FileInnerContentProxy',),
    ('image', 'ImageInnerContentProxy',),
    ('link', 'LinkInnerContentProxy',),
    )

class FieldData:

    def __init__(self, proxy_name, primary_field_name,
                 attached_name, attached_value, attached_expected,
                 ref_type, ref_value, ref_expected,
                 default):

        self.proxy_name = proxy_name
        self.primary_field_name = primary_field_name
        self.attached_name = attached_name
        self.attached_value = attached_value
        self.attached_expected = attached_expected
        self.ref_type = ref_type
        self.ref_value = ref_value
        self.ref_expected = ref_expected
        self.default = default

test_primary_fields = (
    FieldData('file', 'file',
              'attachedFile', 'attached file', 'attached file',
              'File', 'ref file', 'ref file',
              ''),
    FieldData('image', 'image',
              'attachedImage', loadFile('test.gif'), loadFile('test.gif'),
              'Image', loadFile('test.jpg'), loadFile('test.jpg'),
              ''),
    FieldData('link', 'remoteUrl',
              'attachedLink', 'attached link', 'attached link',
              'Link', 'ref link', 'http://nohost/plone/link',
              ''),
    )

file_proxy_values = (
    {'id': 'object_1',
     'title': 'Title 1',
     },
    {'id': 'object_2',
     'title': 'Title 2',
     },
    )


class InnerContentProxyTestCase(BaseTestCase):
    """
    Test all our inner content proxies
    """

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.article = self.addPloneArticle(self.portal, 'article')
        name = 'proxies'
        self.proxies = InnerContentContainer(name)
        self.proxies = self.proxies.__of__(self.portal)
        self.portal._setObject(name, self.proxies)
        for name, portal_type in test_proxies:
            self.proxies.invokeFactory(type_name=portal_type, id=name)

    def testGetPrimaryValue(self):
        self.loginAsPortalOwner()

        #for name, attached_name, attached_value, ref_type, ref_value, default in test_primary_fields:
        for data in test_primary_fields:

            proxy = self.proxies[data.proxy_name]
            primary_field = proxy.getField(data.primary_field_name)
            primary_accessor = primary_field.getAccessor(proxy)

            # Create referenced content
            kwargs = {}
            kwargs[data.primary_field_name] = data.ref_value
            ref_content = self.addContent(data.ref_type, self.portal,
                                          data.proxy_name)
            ref_content.edit(**kwargs)

            # No referenced content, no attached file
            value = primary_accessor()
            self.assertEquals(getattr(value, 'data', value), data.default)

            # Referenced content, no attached file
            kwargs = {}
            kwargs['referencedContent'] = (ref_content.UID(),)
            proxy.edit(**kwargs)
            value = primary_accessor()
            self.assertEquals(getattr(value, 'data', value), data.ref_expected)

            # referenced content, attached file
            kwargs = {}
            kwargs[data.attached_name] = data.attached_value
            proxy.edit(**kwargs)
            value = primary_accessor()
            self.assertEquals(getattr(value, 'data', value),
                              data.attached_expected)

            # No Referenced content, attached file
            kwargs = {}
            kwargs['referencedContent'] = ()
            value = primary_accessor()
            self.assertEquals(getattr(value, 'data', value), data.attached_expected)

        self.logout()

    def testReferenceableInterfaces(self):

        for class_name in test_types_names:
            klass = getattr(pa_proxies, class_name)
            rf_present = hasattr(klass, 'referenceable_interfaces')
            if class_name == 'BaseInnerContentProxy':
                self.failIf(rf_present,
                            "BaseInnerContentProxy must not define 'referenceable_interfaces'")
            else:
                self.failIf(not rf_present, "%s must define 'referenceable_interfaces'" % class_name)
                attr = getattr(klass, 'referenceable_interfaces')
                is_tuple = type(attr) == TupleType
                self.failUnless(is_tuple, "%s.referenceable_interfaces is not a tuple" % class_name)

                if is_tuple:
                    for i in attr:
                        self.failUnless(issubclass(i, Interface),
                            "%s.referenceable_interfaces: %s is not an interface" % (class_name, i.__name__))

    def testgetReferenceablePortalTypes(self):
        article = self.article
        article.setFiles(file_proxy_values)
        fp = article.getFiles()[0]
        rpt = fp.getReferenceablePortalTypes(article.getField('files'))
        self.assertEquals(rpt, ['File'])

    def testRenameAfterCreation(self):
        fields_data = (
            ('LinkInnerContentProxy',
             {'title': 'My nice title',}, 'my-nice-title',
             ),
            ('ImageInnerContentProxy',
             {'title': 'Test renamed after file name',
              'attachedImage_file': openTestFile('test.jpg'),
              },
             'test.jpg'),
            ('ImageInnerContentProxy',
             {'title': 'Test twice the same file name',
              'attachedImage_file': openTestFile('test.jpg'),
              },
             'test-1.jpg'),
            ('FileInnerContentProxy',
             {'title': 'Test file renamed after file name',
              'attachedFile_file': openTestFile('test.jpg'),
              },
             'test-2.jpg'),
            )

        self.loginAsPortalOwner()

        # from http://www.zope.org/Members/shh/ZopeTestCaseWiki/FrequentlyAskedQuestions
        # ensure we have _p_jar to avoid problems with CopySupport (cb_isMoveable)
        import transaction
        transaction.savepoint(optimistic=True)

        for ptype, data, expected_id in fields_data:
            initial_id = self.portal.generateUniqueId(ptype)
            proxy_id = self.proxies.invokeFactory(ptype, initial_id)
            proxy = self.proxies[proxy_id]
            proxy.processForm(values=data)

            self.assertEquals(proxy.getId(), expected_id)

    def test_searcheableText(self):
        """ test circular reference in linkinnerproxy """
        self.loginAsPortalOwner()

        ## article1 reference article2
        article1 = self.article
        article2 = self.addPloneArticle(self.portal, 'article2')
        _createObjectByType('InnerContentContainer',article1,'links')
        _createObjectByType('LinkInnerContentProxy',article1.links, 'link')
        article1.links.link.update(title='Reference to article 2',
                                            description='description2',
                                            referencedContent=article2)
        ## article2 reference artilce1
        _createObjectByType('InnerContentContainer',article2,'links')
        _createObjectByType('LinkInnerContentProxy',article2.links, 'link')
        article2.links.link.update(title='Reference to article 1',
                                    description='description1',
                                    referencedContent=article1)
        self.failUnless('Reference to article 2' in article1.SearchableText() )
        self.failUnless('Reference to article 1' in article2.SearchableText() )





def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(InnerContentProxyTestCase))
    return suite




# -*- coding: utf-8 -*-
## Test case for PloneArticleTool
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test case for all InnerContentProxy classes
"""

__docformat__ = 'restructuredtext'

# Products imports
from Products.PloneArticle.utils import find_classes_implementing_method
from Products.PloneArticle.tests.common import BaseTestCase

class FakeA:
    def dummy(self):
        pass
    
class FakeB:
    def dummy(self):
        pass
    
class FakeC(FakeA, FakeB):
    pass
    

class ArticleUtilsTestCase(BaseTestCase):
    """
    Test functions from utils package
    """
    
    def test_find_classes_implementing_method(self):
        expected_klasses = (FakeA, FakeB)
        klasses = find_classes_implementing_method(FakeC, 'dummy')
        self.assertEqual(expected_klasses, klasses)
    
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(ArticleUtilsTestCase))
    return suite
            




# -*- coding: utf-8 -*-
## Widget for SmartListField
## 
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Widget for SmartListField
"""

__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo

from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.Registry import registerWidget
from Products.Archetypes.Widget import TypesWidget

class SmartListWidget(TypesWidget):
    """"""
    
    _properties = TypesWidget._properties.copy()
    _properties.update({
        'macro' : "pa_smartlistwidget",
        })

    security = ClassSecurityInfo()

    security.declarePublic('getSelectablePortalTypes')
    def getSearchablePortalTypes(self, field, instance):
        """
        Return a Display list for searchable portal_types
        """
        
        pt = getToolByName(instance, 'portal_types')
        allowed_types = field.getAllowedTypes(instance)
        typesList = [pt.getTypeInfo(t) for t in allowed_types]
        
        return instance.createMultiColumnList(typesList, numCols=2,
                                              sort_on='title_or_id')
        
    security.declarePublic('getSelectedTypes')
    def getSelectedTypes(self, search_criterias):
        """
        """
        return search_criterias.get('portal_type', ())

    security.declarePublic('getVisitedUIDs')
    def getVisitedUIDs(self, excluded_uids, referenced_uids):
        """
        """
        visited_uids = dict.fromkeys(referenced_uids)
        visited_uids.update(excluded_uids)
        return visited_uids

    security.declarePublic('getAddableCriterias')
    def getAddableCriterias(self, field, instance, search_criterias):
        """
        """
        return [idx for idx in field.getAllowedCriterias(instance)
                if idx not in ('SearchableText', 'portal_type',)]

    security.declarePublic('getAddedCriterias')
    def getAddedCriterias(self, search_criterias):
        """
        """
        return [idx for idx in search_criterias.keys()
                if idx not in ('SearchableText', 'portal_type',)]
    
    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False):
        """
        collect form values into a dictionnary suitable for field
        """

        fieldName = field.getName()
        value = form.get(fieldName + '_ref_uids', [])
        
        kwargs = {}

        # form record are not dictionnaries, we must create a real one
        kwargs['search_criterias'] = {}
        kwargs['search_criterias'].update(
            form.get(fieldName + '_search_criterias', {})
            )

        kwargs['uids_found'] = form.get(fieldName + '_uids_found', [])
        kwargs['auto_reference'] = form.get(fieldName + '_auto_reference',
                                            False)

        return value, kwargs
            
    
registerWidget(SmartListWidget,
               title='SmartList',
               description=(''),
               used_for=('Products.PloneArticle.field.smartlistfield.SmartListField',)
               )



# -*- coding: utf-8 -*-
## 
## Copyright (C) 2008 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Plone Article migration module from 4.1 betas to 4.1 final releases
"""
__version__ = "$Revision:  $"
# $Source:  $
# $Id:  $
__docformat__ = 'restructuredtext'


# -*- coding: utf-8 -*-
## This module contains all content types used in PloneArticle product
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""
This module is an example of how to inherit from PloneArticle.

It adds a "location" field, which is a simple text field.

"""

__docformat__ = 'restructuredtext'

# Python imports
from copy import deepcopy

# Zope imports
from AccessControl import ClassSecurityInfo

# CMF imports

# Archetypes imports
try:
    from Products.LinguaPlone.public import registerType, Schema, StringField
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import registerType, Schema, StringField

from Products.PloneArticle.content.article import PloneArticle, PloneArticleSchema

from Products.PloneArticle import config

# Inherits schema from PloneArticle
# Copy it to make sure PloneArticle schema is totally independant
ArticleWithLocationSchema = PloneArticleSchema.copy() + Schema((
    StringField(
        'location',
        schemata='misc',
        ),
    ),)

# Inherits actions from PloneArticle
# Copy them to make sure ArticleWithLocation actions are totally independant
ArticleWithLocationActions = deepcopy(PloneArticle.actions)

class ArticleWithLocation(PloneArticle):
    """
        An article with a location field
    """

    # Standard content type setup
    portal_type = meta_type = 'ArticleWithLocation'
    archetype_name = 'Article with location'
    content_icon = 'plonearticle.gif'
    schema = ArticleWithLocationSchema
    typeDescription = 'An article with a location field'
    typeDescMsgId = 'description_edit_articlewithlocation'

    # Set up our views - these are available from the 'display' menu
#    default_view = 'pa_model1'
#    immediate_view = 'pa_model1'
#    suppl_views = (
#        'pa_model2',
#        'pa_model3',
#        'pa_model4',
#        'pa_model5',
#        'pa_model6',
#        'pa_model7',
#        'pa_model8',
#        'pa_model9',
#        'pa_model10',
#        )

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    # Get the standard actions (tabs)
    actions = ArticleWithLocationActions
    security = ClassSecurityInfo()

registerType(ArticleWithLocation, config.PROJECTNAME)



# -*- coding: utf-8 -*-
## Defines utilities to use with article models
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines utilities to use with article models
"""

__docformat__ = 'restructuredtext'

# Zope imports
from zope.interface import implements
from Products.Five import BrowserView
import modelsconfig

def _getThumbUrl(image_url, width, height) :
    """
    Return a thumb url 
    without image.getObject
    """
    return '%s/pa_thumb/imagex%ix%i.jpeg' %(image_url, width, height)
    
def _getImageInfo(image, pwidth, pheight, twidth, theight):
    """
    all informations to display a proxy image
    """   
    image_url = image.absolute_url()
    return dict( thumb_url = _getThumbUrl(image_url, twidth, theight),
                 preview_url = _getThumbUrl(image_url, pwidth, pheight),
                 image_url = image_url,
                 title = image.title_or_id(),
                 description = image.Description(), )    
    

# Products imports
from interface import IPloneArticleModelView, IPloneArticleModel1View, \
     IPloneArticleModel2View, \
     IPloneArticleModel3View, \
     IPloneArticleModel4View, \
     IPloneArticleModel5View, \
     IPloneArticleModel6View, \
     IPloneArticleModel7View, \
     IPloneArticleModel8View, \
     IPloneArticleModel9View, \
     IPloneArticleModel10View, \
     IPloneArticleModel11View

from Products.PloneArticle.interfaces import IFileInnerContentProxy, \
    IImageInnerContentProxy
from Products.PloneArticle.model import registerArticleModel

from Products.PloneArticle.i18n import ModelMessageFactory as _

class PloneArticleModelView(BrowserView):
    """
    No description available
    """
    implements(IPloneArticleModelView)

    template_id = ''
    title = u'Untitled model'
    title_mgsid = 'untitled_model'
    icon = 'pa_default_model.gif'
    description_msgid = 'no_description_available'
    
    def __init__(self, context, request):
      self.context = context
      self.request = request

    def Title(self):
        """Return the translated title of this model"""
        return _(self.title_msgid, default=self.title)
    
    def getDefaultImageSize(self):
        """Returns default dimension(width, height) of an image"""
        
        return 100, 130
        
        
    def getBigImageSize(self):
        """Returns dimension(width, height) for a big image"""
        
        return 500, 350     
           
    
    def getImageUrl(self, proxy):
        """Returns url to access image of an ImageInnerContentProxy object
        """

        if not IImageInnerContentProxy.providedBy(proxy):
            raise ValueError, "Proxy must implements IImageInnerContentProxy"
        
        return proxy.absolute_url()
    
    def getImageBlockStyle(self, proxy):
        """Returns style of block containing the img tag rendering an ImageInnerContentProxy object"""
        
        if not IImageInnerContentProxy.providedBy(proxy):
            raise ValueError, "Proxy must implements IImageInnerContentProxy"
        
        width, height = self.getDefaultImageSize()
        style = 'width: %d; height: %d;' % (width, height)
        return style
        
    def getImageTag(self, proxy):
        """Returns an img tag rendering an ImageInnerContentProxy object
        """
        
        if not IImageInnerContentProxy.providedBy(proxy):
            raise ValueError, "Proxy must implements IImageInnerContentProxy"
        
        width, height = self.getDefaultImageSize()
        params={}
        params['width']=width
        params['height']=height
        # use description as title when possible
        legend=proxy.Description()
        if legend :
          params['title']=legend
        return proxy.tag(**params)
        
    def getBigImageTag(self, proxy):
        """Returns a big img tag rendering an ImageInnerContentProxy object
        """
        
        if not IImageInnerContentProxy.providedBy(proxy):
            raise ValueError, "Proxy must implements IImageInnerContentProxy"
        
        width, height = self.getBigImageSize()
        return proxy.tag(width=width, height=height)       

    def getImageTile(self, proxy):
        """Returns a 16px icon from an ImageInnerContentProxy object
        """
        
        if not IImageInnerContentProxy.providedBy(proxy):
            raise ValueError, "Proxy must implements IImageInnerContentProxy"
        
        return proxy.tag(width=16, height=16)    
        

    def getSmallImage4Crop(self, proxy):
        """Returns at least a 50*50px img tag from an ImageInnerContentProxy object
        """
        
        if not IImageInnerContentProxy.providedBy(proxy):
            raise ValueError, "Proxy must implements IImageInnerContentProxy"
        
        params={} 
        params['maximizeTo'] = 50
        params['maxRatio'] = 1   
        legend=proxy.Title()
        if legend :
            params['title']=legend
        return proxy.tag(**params)        
        
    def getImage4Crop(self, proxy):
        """Returns at least a 250*250px img tag from an ImageInnerContentProxy object
        """
        
        if not IImageInnerContentProxy.providedBy(proxy):
            raise ValueError, "Proxy must implements IImageInnerContentProxy"
        
        params={}
        params['maximizeTo']=250
        params['maxRatio'] = 1   
        legend=proxy.Title()
        if legend :
            params['title']=legend
        return proxy.tag(**params)              

    def getFileUrl(self, proxy):
        """Returns url to access file of an FileInnerContentProxy object
        """
        
        if not IFileInnerContentProxy.providedBy(proxy):
            raise ValueError, "Proxy must implements IImageInnerContentProxy"
        
        return proxy.absolute_url()
        
        
    

class PloneArticleModel1View(PloneArticleModelView):
    """
    Images are displayed on the right, while attachments are on the bottom of
    the document.

    This is a simple layout that can be used for documentation, for example.
    """
    implements(IPloneArticleModel1View)

    template_id = 'pa_model1'
    title = u'Documentation model'
    title_msgid = 'model1_title'
    icon = 'pa_model1.gif'
    description_msgid = 'model1_description'

registerArticleModel(PloneArticleModel1View)

class PloneArticleModel2View(PloneArticleModelView):
    """
    The first image is displayed in the text header.

    Other images are displayed on the right, while attachments are on the bottom
    of the document.
    
    This is a simple layout that can be used for end-user articles, for example.
    """
    implements(IPloneArticleModel2View)    

    template_id = 'pa_model2'
    title = u'Newspaper article model'
    title_msgid = 'model2_title'
    description_msgid = 'model2_description'
    icon = 'pa_model2.gif'

    def __init__(self, context, request):
        super(PloneArticleModelView, self).__init__(context, request)
        self.images = context.getImages()

    def getFirstImage(self):
        return len(self.images) > 0 and self.images[0] or None

    def getOtherImages(self):
        return self.images[1:]
    
registerArticleModel(PloneArticleModel2View)

class PloneArticleModel3View(PloneArticleModelView):
    """
    Shrinked images are placed after the description and the text.
    """
    implements(IPloneArticleModel3View)

    template_id = 'pa_model3'
    title = u'Small images'
    title_msgid = 'model3_title'
    description_msgid = 'model3_description'
    icon = 'pa_model3.gif'
    
registerArticleModel(PloneArticleModel3View)

class PloneArticleModel4View(PloneArticleModelView):
    """
    The description and the text are displayed first.
    Big images are displayed after the text.
    """
    implements(IPloneArticleModel4View)

    template_id = 'pa_model4'
    title = u'Big images'
    title_msgid = 'model4_title'
    description_msgid = 'model4_description'
    icon = 'pa_model4.gif'
    
registerArticleModel(PloneArticleModel4View)

class PloneArticleModel5View(PloneArticleModelView):
    """
    The first two images are displayed inline first. 
    Then comes the body of the article.
    All other images that illustrate the article come after and are displayed
    inline two by two.
    """
    implements(IPloneArticleModel5View)

    template_id = 'pa_model5'
    title = u'Small images separated by text'
    title_msgid = 'model5_title'
    description_msgid = 'model5_description'
    icon = 'pa_model5.gif'

    def __init__(self, context, request):
        super(PloneArticleModelView, self).__init__(context, request)
        self.images = context.getImages()

    def getHeadingImages(self):
        return self.images[:2]

    def getBodyImages(self):
        return self.images[2:]
    
registerArticleModel(PloneArticleModel5View)

class PloneArticleModel6View(PloneArticleModel2View):
    """
    The first image is big and displayed first.
    Other images are displayed after the article text.
    """
    implements(IPloneArticleModel6View)

    template_id = 'pa_model6'
    title = u'Big images separated by text'
    title_msgid = 'model6_title'
    description_msgid = 'model6_description'
    icon = 'pa_model6.gif'

registerArticleModel(PloneArticleModel6View)

class PloneArticleModel7View(PloneArticleModel2View):
    """
    The first image is big and followed by the body of the article.
    All other images come after in shrinked size.
    """
    implements(IPloneArticleModel7View)

    template_id = 'pa_model7'
    title = u'One big image and a lot of small'
    title_msgid = 'model7_title'
    description_msgid = 'model7_description'
    icon = 'pa_model7.gif'

registerArticleModel(PloneArticleModel7View)

class PloneArticleModel8View(PloneArticleModel2View):
    """
    This model diplays first an image (in its normal size), followed by the body
    of the article.
    All other images are shrinked and come after the text.
    The attachments are displayed on the left of the article.
    """
    implements(IPloneArticleModel8View)

    template_id = 'pa_model8'
    title = u'One big image and lots of small and side attachments'
    title_msgid = 'model8_title'
    description_msgid = 'model8_description'
    icon = 'pa_model8.gif'

registerArticleModel(PloneArticleModel8View)

class PloneArticleModel9View(PloneArticleModelView):
    """
    This model diplays images icons, files and links at bottom of page
    """
    implements(IPloneArticleModel9View)

    template_id = 'pa_model9'
    title = u'images icons, files and links at page bottom'
    title_msgid = 'model9_title'
    description_msgid = 'model9_description'
    icon = 'pa_model9.gif'

registerArticleModel(PloneArticleModel9View)

class PloneArticleModel10View(PloneArticleModelView):
    """
    4 first images displayed at left and cropped in a 50*50px square
    Zoom on first image displayed cropped in a 250*250px square
    Others images displayed at page bottom as icons
    """
    implements(IPloneArticleModel10View)

    template_id = 'pa_model10'
    title = u'Square cropped images'
    title_msgid = 'model10_title'
    description_msgid = 'model10_description'
    icon = 'pa_model10.gif'

    def __init__(self, context, request):
        super(PloneArticleModelView, self).__init__(context, request)
        self.images = context.getImages()
        
    def getFirstImage(self):
        return len(self.images) > 0 and self.images[0] or None        

    def getHeadingImages(self):
        return self.images[:4]
    
registerArticleModel(PloneArticleModel10View)


class PloneArticleModel11View(PloneArticleModelView):
    """
    Ajax viewer for images
    """
    implements(IPloneArticleModel11View)

    template_id = 'pa_model11'
    title = u'Ajax slide viewer'
    title_msgid = 'model11_title'
    description_msgid = 'model11_description'
    icon = 'pa_model11.gif'

    def __init__(self, context, request):
        super(PloneArticleModelView, self).__init__(context, request)
        self.images = context.getImages()
        self.maxwidth = modelsconfig.MAXWIDTH
        self.maxheight = modelsconfig.MAXHEIGHT    
        self.thumbmaxwidth = modelsconfig.THUMBMAXWIDTH
        self.thumbmaxheight = modelsconfig.THUMBMAXHEIGHT     
        self.disablestyles = modelsconfig.DISABLESTYLES   
        self.float = modelsconfig.FLOAT      
        self.bgcolor = modelsconfig.BGCOLOR      
        self.bordercolor = modelsconfig.BORDERCOLOR 
        self.margin = modelsconfig.MARGIN        
        self.controlsorientation = modelsconfig.CONTROLSORIENTATION    
        self.verticalcontrolsposition = modelsconfig.VERTICALCONTROLSPOSITION 
        self.horizontalcontrolsposition = modelsconfig.HORIZONTALCONTROLSPOSITION   


    def getPlayerImages(self):
        """
        tags & image's urls for player
        """      
        images = []
        all_views = self.images
        for image in all_views :
            imageInfo = _getImageInfo (image, self.maxwidth, self.maxheight,
                                       self.thumbmaxwidth, self.thumbmaxheight)
            images.append(imageInfo)
                               
        return len(all_views) > 0 and images or None
        

        
    def _getGlobalStyle(self):
        """
        Player global style
        Return only width if global styles disabled
        """              
        if self.disablestyles :
            return 'width: %ipx' %self.maxwidth
        return '''width: %ipx; 
                  clear: %s;
                  float: %s;
                  margin: %s;
                  border-color:%s;
                  background-color: %s''' %(self.maxwidth, self.float, self.float, self.margin,
                                            self.bordercolor, self.bgcolor ) 
                                            
    def _getGlobalClass(self):
        """
        Player global class (vertical or horizontal, vertical buttons on left or right)
        """              
        
        return '%sPlayer %sVButtons %sHButtons' %(self.controlsorientation, self.verticalcontrolsposition,
                                                  self.horizontalcontrolsposition )                                                
        
    def getPlayerStyle(self):
        """
        Player width & height & global style
        """      
        
        return {'maxwidth' : self.maxwidth,
                'maxheight' : self.maxheight,
                'thumbmaxwidth' : self.thumbmaxwidth,
                'thumbmaxheight' : self.thumbmaxheight,  
                'globalstyles': self._getGlobalStyle(),  
                'globalclass': self._getGlobalClass(),}
                
    def getPlayerFirstImage(self):
        """
        tags & image's urls for first image player
        displayed even when javascript is disabled
        """                      
        images = self.images
        return len(images) > 0 and _getImageInfo (images[0], self.maxwidth, self.maxheight,
                                       self.thumbmaxwidth, self.thumbmaxheight) or None

    
registerArticleModel(PloneArticleModel11View)



## Script (Python) "pa_innerfile_reference"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=field_name,  uid, widget_index=None
##title=
##

return context.pa_innercontent_reference(field_name, uid, widget_index)


## Script (Python) "pa_innercontent_reference"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=field_name, uid, widget_index=None
##title=
##
from Products.CMFCore.utils import getToolByName

request = context.REQUEST
pa_tool = getToolByName(context, 'plonearticle_tool')
reference_tool = getToolByName(context, 'reference_catalog')
obj = reference_tool.lookupObject(uid)

if not obj:
    return context.standard_error_message(error_type=404,
     error_message="The link you followed appears to be broken")

field = context.getField(field_name)
tmp_content = field.getTemporaryInnerContent(context)
accessor = field.getAccessor(context)
existing_ids = [c.getId() for c in accessor()]
mutator = field.getMutator(context)
template = context.restrictedTraverse(path='pa_temporary_innerimage_widget')

# help utranslate to write with proper encoding
utils = getToolByName(context, 'plone_utils')
charset = utils.getSiteEncoding()
request.RESPONSE.setHeader('Content-Type', 'text/html; charset=%s' % charset)

# Reference new content
try:
    mutator(({"id": tmp_content.getId(),
              "title": obj.title_or_id(),
              "description": obj.Description(),
              "referencedContent": uid},),
             update=True)
except:
    exception = context.plone_utils.exceptionString()
    message = "Referenced content error: %s" % exception
    return template(from_upload=False, skip_widget=True, fail_reason=message)


# FIXME: This use case cannot happen. Maybe delete it
# Check if content has been added to article
new_inner_contents = [c for c in accessor()
                     if c.getId() not in existing_ids]

if len(new_inner_contents)!=1:
    exception = "Inner content has not been added"
    message = "Referenced content error: %s" % exception
    return template(from_upload=False, skip_widget=True, fail_reason=message)

inner_content = new_inner_contents[0]

# Render template
widget = field.widget
if widget_index is None:
    widget_index = widget.generateWidgetIndex()   
widget_index = int(widget_index)

return template(from_upload=False,
                inner_content=inner_content,
                field=field,
                field_name=field_name,
                widget=widget,
                widget_index=widget_index)


# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: lfm_article.py 5834 2007-02-08 13:16:33Z roeder $
from funittest import logical
from funittest import interpreter
from funittest import physical
from funittest import dataprovider
from funittest import register_lfm
from funittest import Testable
import random
import os

class Article(Testable):
    """
    Articles can have three types of attachments:
    
    files, images and links
    
    These attachments can be uploaded in the same way.
    """
    def add_file(self, file, postcondition="success"):
        interpreter.annotate("Add a file to the article")
        physical.cmfplone.tab.access("edit")
        physical.cmfplone.schemata.access("files")
        physical.plonearticle.attachment_upload.add(file, postcondition)

    def add_image(self, image, postcondition="success"):
        interpreter.annotate("Add an image to the article")
        physical.cmfplone.tab.access("edit")
        physical.cmfplone.schemata.access("images")
        physical.plonearticle.attachment_upload.add(image, postcondition)

    def add_multiple_images(self, images):
        start=1
        for image in images:
            if start:
                physical.cmfplone.tab.access("edit")
                physical.cmfplone.schemata.access("images")
                physical.plonearticle.attachment_upload.access()
                physical.plonearticle.attachment_upload.fill(image)
                physical.plonearticle.attachment_upload.save(image)
                start=0
            else:                
                physical.plonearticle.attachment_upload.fill(image)
                physical.plonearticle.attachment_upload.save(image)
        # Leave upload
        physical.plonearticle.attachment_upload.cancel()     
        
    def add_link(self, link, postcondition="success"):
        interpreter.annotate("Add a link to the article")
        physical.cmfplone.tab.access("edit")
        physical.plonearticle.attachment_upload.access_links()
        physical.plonearticle.attachment_upload.add(link, postcondition)

    def save_article(self, article):
        logical.cmfplone.content.save(article)            

    def browse_file(self, file):
        interpreter.annotate("Browse for a file to be added to the article")
        physical.cmfplone.tab.access("edit")
        physical.cmfplone.schemata.access("files")
        physical.plonearticle.attachmentbrowse.access()
        # To be continued...
        
    def browse_image(self, image):
        interpreter.annotate("Browse for an image to be added to the article")
        physical.cmfplone.tab.access("edit")
        physical.cmfplone.schemata.access("images")
        physical.plonearticle.attachmentbrowse.access()
        # To be continued...

    def browse_link(self, link):
        interpreter.annotate("Browse for a link to be added to the article")
        physical.cmfplone.tab.access("edit")
        physical.cmfplone.schemata.access("links")
        physical.plonearticle.attachmentbrowse.access()
        # To be continued...

    def delete_file(self, file):
        interpreter.annotate("Delete a file from the article")
        physical.cmfplone.tab.access("edit")
        physical.cmfplone.schemata.access("files")
        # To be continued        
        
register_lfm("PloneArticle", Article())



## Script (Python) "pa_create_inner_content_widget"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=field_name, widget_index=None
##title=Returns html code of a new inner content widget
##
##
field = context.getField(field_name)
widget = field.widget
inner_content = field.getTemporaryInnerContent(context)

if widget_index is None:
    widget_index = widget.generateWidgetIndex()
    
widget_index = int(widget_index)

# Render template
template = context.restrictedTraverse(path='pa_new_inner_content_widget')
return template(field_name=field_name, field=field, widget=widget,
                widget_index=widget_index, inner_content=inner_content)



# -*- coding: utf-8 -*-
## Product description
## 
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Multipage views
"""

__docformat__ = 'restructuredtext'

# Zope imports
from zope.interface import Interface, implements
from Products.Five import BrowserView
from Acquisition import aq_inner
from plone.app.layout.navigation.defaultpage import DefaultPage
from interface import IPloneArticleMultipageView
from Products.PloneArticle.interfaces import IPloneArticle
    
# class MultiPageDefaultPage(DefaultPage):
#     def getDefaultPage(self, context_=None):
#         """
#         Override Plone default implementation:
#         Use multipage default page
#         """
        
#         context = aq_inner(self.context)
#         return context.getDefaultPage()


class NonMultipageContext(BrowserView):
    """
    View for generic article containers (folders, ...)
    """
    implements(IPloneArticleMultipageView)

    def isMultipage(self):
        return False

    def getPages(self):
        raise ValueError, "not in a multipage article"

    def getTOCItems(self, context=None):
        raise ValueError, "not in a multipage article"

    def getPreviousPage(self, context):
        raise ValueError, "not in a multipage article"

    def getNextPage(self, context):
        raise ValueError, "not in a multipage article"

class PloneArticleMultipageView(BrowserView):
    """
    View for PloneArticleMultipage
    """
    implements(IPloneArticleMultipageView)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.pages = None

    ### IPloneArticleMultipageView #####################################

    def isMultipage(self):
        return True

    def getPages(self):
        pages = self.pages
        if pages is None:
            pages = self.pages = self.context.getPages()
        return pages

    def getTOCItems(self, context=None):
        
        current_article_id = (context is not None) and context.getId()
        pages = self.getPages()
        pages_list = []
        
        for page in pages:
            page_id = page.getId
            is_selected = current_article_id and (page_id == current_article_id)
            item = {
                'id': page_id,
                'title': page.Title,
                'url': page.getURL(),
                'selected': is_selected,
                }
            pages_list.append(item)

        return pages_list

    def getPreviousPage(self, context):
        context_id = context.getId()
        pages = self.getPages()
        idx = self._getPageIndex(pages, context_id)

        # idx is None or 0
        if not idx:
            return None
        
        return pages[idx - 1]

    def getNextPage(self, context):
        context_id = context.getId()
        pages = self.getPages()
        idx = self._getPageIndex(pages, context_id)

        if idx is None or (idx+1 >= len(pages)):
            return None
        return pages[idx + 1]

    ### private methods ################################################

    def _getPageIndex(self, pages, page_id):
        pages_id = [p.getId for p in pages]
        try:
            idx = pages_id.index(page_id)
        except ValueError:
            return None
        return idx



# -*- coding: utf-8 -*-
## PloneArticle product provides content with files, images, links
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module contains utilities, helpers
"""

from Products.CMFCore.utils import getToolByName
from Products.PloneArticle.interfaces import \
     IBaseInnerContentProxy as z2_IBaseInnerContentProxy


def find_classes_implementing_method(klass, method_name):
    """Returns a list of classes implementing a given method.

    @param method_name: Name of the method we are searching for
    """

    if method_name in klass.__dict__.keys():
        return (klass,)

    base_klasses = klass.__bases__
    result = []
    for base_klass in base_klasses:
        result.extend(find_classes_implementing_method(base_klass, method_name))

    return tuple(result)

def getAllAvailableReferenceableTypes(context, klass):
    ## XXX perhaps check that klass is an instance of something correct ?
    interfaces = klass.getReferenceableInterfaces()

    at = getToolByName(context, 'archetype_tool')
    proxy_types = at.listPortalTypesWithInterfaces([z2_IBaseInnerContentProxy,])
    proxy_types = [fti.getId() for fti in proxy_types]

    ## ct are content types implemented by these interfaces but
    ## which are not proxies
    ct = at.listPortalTypesWithInterfaces(interfaces)
    ct = [c_fti.getId() for c_fti in ct]
    ct = [pt for pt in ct if not pt in proxy_types]
    return ct





# -*- coding: utf-8 -*-
## Test case for all InnerContentWidget classes
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test case for all InnerContentWidget classes
"""

__docformat__ = 'restructuredtext'

# Products imports
from Products.PloneArticle.tests.common import BaseTestCase

test_fieldnames = ('files', 'images', 'links',)

form = {
    'files_innercontent000000_id': 'file_1',
    'files_innercontent000000_title': 'File 1',
    'files_innercontent000001_id': 'file_2',
    'files_innercontent000001_title': 'File 2',
    'files_innercontent_position': ('000000', '000001'),
    'images_innercontent000000_id': 'image_1',
    'images_innercontent000000_title': 'Image 1',
    'images_innercontent_position': ('000000',),
    'links_innercontent000000_id': 'link_1',
    'links_innercontent000000_title': 'Link 1',
    'links_innercontent000001_id': 'link_2',
    'links_innercontent000001_title': 'Link 2',
    'links_innercontent_position': ('000001', '000000'),
}

expected_values = {
    'files': (
        (
            {'id': ('file_1', {}),
             'title': ('File 1', {}),
             },
            {'id': ('file_2', {}),
             'title': ('File 2', {}),
             },
        ), 
        {},
    ),
    'images': (
        (
            {'id': ('image_1', {}),
             'title': ('Image 1', {}),
             },
        ), 
        {},
    ),
    'links': (
        (
            {'id': ('link_2', {}),
             'title': ('Link 2', {}),
             },
            {'id': ('link_1', {}),
             'title': ('Link 1', {}),
             },
        ), 
        {},
    ),
}

tests = []

class InnerContentWidgetTestCase(BaseTestCase):
    """
    This class contains all test cases for all InnerContentWidget classes
    """

    def afterSetUp(self):
        """ """
        self.loginAsPortalOwner()
        self.article = self.addPloneArticle(self.portal, 'article')
        self.logout()
        
    def test_processing(self):
        """ """
        article = self.article
        
        for name in test_fieldnames:
            field = article.getField(name)
            widget = field.widget
            value = widget.process_form(article, field, form)
            expected_value = expected_values[name]
            self.assertEquals(len(value), 2)
            self.assertEquals(value[1], expected_value[1])
            
            # Test dictionnaries, so test each key
            items = value[0]
            expected_items = expected_value[0]
            self.assertEquals(len(items), len(expected_items))
            for index in range(0, len(items)):
                result_items = items[index].items()
                result_items.sort()
                expected = expected_items[index].items()
                expected.sort()
                self.assertEquals(result_items, expected)

    def testMakeWidgetId(self):
        """"""
        article = self.article
        field = article.getField('images')
        widget = field.widget

        expected = 'myFieldName-innercontent000042'
        result = widget.makeWidgetId('myFieldName', 42)
        self.assertEqual(result, expected)

    def testWidgetFieldAssocs(self):
        assocs = (
            ('FileInnerContentField', 'FileInnerContentWidget'),
            ('ImageInnerContentField', 'ImageInnerContentWidget'),
            ('LinkInnerContentField', 'LinkInnerContentWidget'),
            )

        product = 'Products.PloneArticle.'

        from Products.Archetypes.Registry import fieldDescriptionRegistry
        from Products.Archetypes.Registry import widgetDescriptionRegistry
        
        for field, widget in assocs:
            module = field[:-5].lower()
            field_path = product + 'field.' + module + '.' + field
            desc = fieldDescriptionRegistry.get(field_path, None)
            self.failIf(desc is None, "%s is not registered" % field_path)

            widget_path = product + 'widget.' + module + '.' + widget
            self.assert_(
                widgetDescriptionRegistry.get(widget_path, None) is not None,
                "Widget %s is not properly registered" % widget
                )
            self.assert_(widget_path in desc.allowed_widgets(),
                        "Widget %s is not listed in allowed_widget for %s"
                        % (widget_path, field_path))
            
tests.append(InnerContentWidgetTestCase)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    for test in tests:
        suite.addTest(makeSuite(test))
    return suite



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from funittest import interpreter
from funittest import register_pom

class Attachment_Upload:
    """
    When the user edits an article, he can access the different attachments,
    image, link and file.
    The user can click on the upload button to add an attachment to the
    article.
    The upload form is the same for the different attachment types, so
    this model is designed to be reusable.
    """
    
    def add(self, attach, postcondition="success"):
        """
        The user is inside the attachment view of an article. He wants to
        upload an attachment. The procedure is always the same for images,
        files and links.
        
        - attach: The attachment to add (image, link or file)
        - postcondition: "success" or "error" depending on the expectation
        """        
        # The user fills in the upload form
        self.fill(attach)
        # The user saves the upload form
        self.save(attach, postcondition)
        # The user has to use the cancel button if he doesn't want to add any more attachments
        self.cancel()    

    def access_links(self):
        """
        Access the upload view by clicking on the upload link.
        """
        interpreter.click("//div[@id='archetypes-fieldname-links']//a[@id='plone-article-upload-link']")
        interpreter.wait_for_condition("selenium.isElementPresent('new_file_btn_ok');",10000)    

    def access_images(self):
        """
        Access the upload view by clicking on the upload link.
        """
        interpreter.click("//div[@id='archetypes-fieldname-images']//a[@id='plone-article-upload-link']")
        interpreter.wait_for_condition("selenium.isElementPresent('new_file_btn_ok');",10000)    

    def access_files(self):
        """
        Access the upload view by clicking on the upload link.
        """
        interpreter.click("//div[@id='archetypes-fieldname-files']//a[@id='plone-article-upload-link']")
        interpreter.wait_for_condition("selenium.isElementPresent('new_file_btn_ok');",10000)    
        
    def fill(self, form):
        """
        Fill in the attachment form generically.
        """
        for key, value in form.items():
            interpreter.type(key, value)

    def save(self, attach, postcondition="success"):        
        """
        Save the attachment.
        
        Postcondition can be "success" or "error"
        """
        interpreter.click("new_file_btn_ok")
        if postcondition=="success":
            interpreter.wait_for_condition("selenium.isElementPresent('upload-status-success');",10000)    
        elif postcondition=="error":
            interpreter.wait_for_condition("selenium.isElementPresent('upload-status-error');",10000)
            # In case of an error, the title and description should not be lost
            interpreter.failUnless(interpreter.get_value("//input[@id='new_file_title']")==attach['new_file_title'])            
            interpreter.failUnless(interpreter.get_value("//input[@id='new_file_description']")==attach['new_file_description'])            
            
    def cancel(self):
        """
        Cancel the upload form, and return to the attachment form.
        """
        interpreter.click("btnCancel")

register_pom("PloneArticle", Attachment_Upload())



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: test_add_image.py 5836 2007-02-08 13:17:49Z roeder $
import unittest
from funittest import logical
from funittest import physical
from funittest import dataprovider
from funittest import precondition
from funittest import interpreter
from funittest import TestCase

import os

class TestAddImage(TestCase):

    def setUp(self):    
        TestCase.setUp(self)
        interpreter.open("")        
        user=dataprovider.cmfplone.user.get('sampleadmin')
        logical.cmfplone.application.change_user(user)        
        self.article = dataprovider.plonearticle.article.get("Article 1")
        precondition.plonearticle.article.existing_article(self.article)

    def test_add_image(self):
        "Add an image to an article and save the article"
        interpreter.annotate("Test: Add an image to the article and save the article")
        image = dataprovider.plonearticle.articleimage.get("Image 1")
        # Normal attachment upload
        logical.plonearticle.article.add_image(image)
        logical.plonearticle.article.save_article(self.article)

    def test_add_non_existing_image(self):
        "Try adding a non-existing image to an article"
        interpreter.annotate("Test: Try adding a non-existing image to an article")
        image = dataprovider.plonearticle.articleimage.get("Non-existing Image")
        physical.cmfplone.tab.access("edit")
        physical.cmfplone.schemata.access("images")
        physical.plonearticle.attachment_upload.access()
        physical.plonearticle.attachment_upload.fill(image)
        physical.plonearticle.attachment_upload.save(image, "error")        
        interpreter.verifyTextPresent("Image upload error: cannot identify image file")
        # Due to differences in the *chrome mode, the following error is not produced:
        # interpreter.verifyTextPresent("Error: Zero file size")
        physical.plonearticle.attachment_upload.cancel()  
        logical.plonearticle.article.save_article(self.article)
        
    def test_cancel_adding_an_image(self):
        "Cancel adding an image"
        interpreter.annotate("Test: Add an image to the article and cancel")
        image = dataprovider.plonearticle.articleimage.get("Image 1")
        physical.cmfplone.tab.access("edit")
        physical.cmfplone.schemata.access("images")
        physical.plonearticle.attachment_upload.access()
        physical.plonearticle.attachment_upload.fill(image)
        # Not saving here.
        physical.plonearticle.attachment_upload.cancel()  
        logical.plonearticle.article.save_article(self.article)

    def test_add_two_images_and_change_position(self):
        "Add two images and change their position"
        interpreter.annotate("Test: Add two images and change their position")

        images = []
        images.append(dataprovider.plonearticle.articleimage.get("Image 1"))
        images.append(dataprovider.plonearticle.articleimage.get("Image 2"))
        logical.plonearticle.article.add_multiple_images(images)

        # Change position of images       
        interpreter.click("//a[@class='file_move_down'][1]")
        interpreter.click("//a[@class='file_move_down'][1]")
        logical.plonearticle.article.save_article(self.article)
        
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
## This module contains all content types used in PloneArticle product
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module contains all content types used in PloneArticle product
"""

__docformat__ = 'restructuredtext'

# Inner content interfaces
from innercontentproxy import IBaseInnerContentProxy, IFileInnerContentProxy, \
    IImageInnerContentProxy, ILinkInnerContentProxy

from innercontentfield import IBaseInnerContentField, IFileInnerContentField, \
    IImageInnerContentField, ILinkInnerContentField, IBaseInnerContent, \
    IInnerContentContainer

from innercontentwidget import IBaseInnerContentWidget, \
    IFileInnerContentWidget, IImageInnerContentWidget, ILinkInnerContentWidget

from article import IPloneArticle, IPloneArticleMultiPage, IPloneArticleTool

from smartlistfield import ISmartListField

# We need this as long as we will support both Plone 2.1 & 2.5 : 2.1 does not
# provide a Z3 interface for INonStructuralFolder, but Plone 2.5 require that it
# is implemented with a Z3 one to take it into account (see implement.zcml)
try:
    from Products.CMFPlone.interfaces.structure import INonStructuralFolder
except ImportError:
    from Products.CMFPlone.interfaces.NonStructuralFolder import \
         INonStructuralFolder as z2_INonStructuralFolder
    from Products.Five.bridge import fromZ2Interface
    INonStructuralFolder = fromZ2Interface(z2_INonStructuralFolder)



# -*- coding: utf-8 -*-
## 
## Copyright (C) 2008 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Plone Article migration module from 4.0 to 4.1 betas
"""
__version__ = "$Revision:  $"
# $Source:  $
# $Id:  $
__docformat__ = 'restructuredtext'

from Products.PloneArticle.migration import applyArticles


def stable40x_410beta1(portal):

    out = []

    def modifyMetaTypes(article):
        # Change meta_type of xxxinnercontentproxy subobjects
        old_mts_to_news = {
            'FileContentInnerProxy': 'FileInnerContentProxy',
            'ImageContentInnerProxy': 'ImageInnerContentProxy',
            'LinkContentInnerProxy': 'LinkInnerContentProxy'
            }
        old_mts = old_mts_to_news.keys()
        for icc in article.objectValues(spec='InnerContentContainer'):
            for item in icc.objectValues():
                mt = item.meta_type
                if mt in old_mts:
                    item.meta_type = old_mts_to_news[mt]
        return

    applyArticles(portal, modifyMetaTypes)
    out.append("Changed meta_type of inner items")
    return out



# -*- coding: utf-8 -*-
## Copyright (C) 2005-2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module contains the tool of PloneArticle product
"""

# Python imports
import os, sys, traceback
from StringIO import StringIO
import logging
from types import StringType, ListType, TupleType
import re
from cgi import escape

# Zope imports
import transaction
from Globals import InitializeClass
from ZODB.POSException import ConflictError
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from OFS.Image import File, Image
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

# CMF imports
from Products.CMFCore.utils import UniqueObject, getToolByName, registerToolInterface
from Products.CMFCore import permissions as CCP

# Plone imports
from Products.CMFPlone import PloneMessageFactory

# Archetypes imports
from Products.Archetypes.public import MinimalSchema, Schema, ImageField, BaseUnit

# Products imports
from Products.PloneArticle import LOG
from Products.PloneArticle.config import PLONEARTICLE_TOOL
from Products.PloneArticle.interfaces import IPloneArticle, IPloneArticleTool, IBaseInnerContentProxy
from Products.PloneArticle.browser.interface import IPloneArticleModelView
from Products.PloneArticle.proxy import BaseInnerContentProxy
from Products.PloneArticle.field import BaseInnerContentField
from Products.PloneArticle.model import getModelRegistry
from Products.PloneArticle import utils
from Products.PloneArticle.i18n import ArticleMessageFactory

# Check AttachmentField product installation
USE_ATTACHMENT_FIELD = True

try:
    from Products.AttachmentField import AttachmentField
except:
    USE_ATTACHMENT_FIELD = False

# Parse thumbnail id
THUMBNAIL_RE = re.compile(r'(?P<field_name>[a-zA-Z0-9]*)x(?P<width>\d*)x(?P<height>\d*).(?P<ext>[a-zA-Z0-9]*)')

try:
    import PIL.Image
except ImportError:
    # no PIL, no scaled versions!
    HAS_PIL = False
else:
    HAS_PIL = True


_upgradePaths = {}

def registerUpgradePath(oldversion, newversion, function):
    """ Basic register func """
    _upgradePaths[oldversion.lower()] = [newversion.lower(), function]


class PloneArticleTool(UniqueObject, SimpleItem):
    """
    Tool for PloneArticle
    """

    plone_tool = True
    id = PLONEARTICLE_TOOL
    title = "Misc utilities for PloneArticle"
    meta_type = 'PloneArticleTool'

    _numversion = ()
    _version = ''

    manage_options = SimpleItem.manage_options

    security = ClassSecurityInfo()

    # structure:{'PloneArticle': {'image': ['Image'], ...}, ...}
    referenceable_types = {}

    migrationWwwPath = os.path.join(os.path.dirname(__file__), 'www', 'migration')

    manage_options += (
        ({'label' : 'Migration',
          'action' : 'migration'
          },)
    )

    migration = PageTemplateFile(migrationWwwPath)
    security.declareProtected(CCP.ManagePortal, 'migration')


    security.declarePublic('listArticleImplementers')
    def listArticleImplementers(self):
        """
        Return all portal types which support IPloneArticle
        """
        at = getToolByName(self, 'archetype_tool')
        return at.listPortalTypesWithInterfaces((IPloneArticle,))

    security.declarePublic('getModelRegistry')
    def getModelRegistry(self):
        """
        """
        return getModelRegistry()

    security.declarePublic('getModelIcon')
    def getModelIcon(self, model_id):
        """
        """
        registry = getModelRegistry()
        model = registry.get(model_id, None)
        if model is None:
            return "pa_default_model.gif"
        return model.icon

    security.declarePublic('getModelsForType')
    def getEnabledModelsForType(self, portal_type):
        """
        Return the set of templates availables as views for portal_type, and the
        default_view .

        @param portal_type: a string or a portal_type object
        """
        if isinstance(portal_type, basestring):
            pt = getToolByName(self, 'portal_types')
            portal_type = pt.getTypeInfo(portal_type)

        return (portal_type.view_methods, portal_type.default_view)

    security.declareProtected(CCP.ManagePortal, 'setEnabledModelsForType')
    def setEnabledModelsForType(self, portal_type, templates,
                                default_view=None):
        """
        Change the set of templates availables as views for portal_type.

        @param portal_type: a string or a portal_type object
        """
        if (not type(templates) in (ListType, TupleType)) \
               or len(templates) == 0:
            return

        if isinstance(portal_type, basestring):
            pt = getToolByName(self, 'portal_types')
            portal_type = pt.getTypeInfo(portal_type)

        if default_view not in templates:
            # if we don't do that CMFDynamicviewFTI will raise ValueError
            # we must give a valid default view
            default_view = templates[0]

        portal_type.manage_changeProperties(view_methods=tuple(templates),
                                            default_view=default_view)


    security.declareProtected(CCP.ManagePortal, 'setOptionsForType')
    def setOptionsForType(self, portal_type_name, field_name, arg_name, **args):
        """
            Set various options in FTIs properties.
        """
        pt = getToolByName(self, 'portal_types')
        portal_type = pt.getTypeInfo(portal_type_name)
        portal_type.manage_changeProperties(**args)
        updates = {
            portal_type_name: {
                field_name: args[arg_name]
            }
        }
        self.updateReferenceableTypes(updates=updates)


    security.declarePublic('getAllAvailableReferenceableImageTypes')
    def getAllAvailableReferenceableImageTypes(self):
        from Products.PloneArticle.proxy.imageinnercontent import ImageInnerContentProxy
        return utils.getAllAvailableReferenceableTypes(self, ImageInnerContentProxy)

    security.declarePublic('getAllAvailableReferenceableLinkTypes')
    def getAllAvailableReferenceableLinkTypes(self):
        from Products.PloneArticle.proxy.linkinnercontent import LinkInnerContentProxy
        return utils.getAllAvailableReferenceableTypes(self, LinkInnerContentProxy)

    security.declarePublic('getAllAvailableReferenceableAttachmentTypes')
    def getAllAvailableReferenceableAttachmentTypes(self):
        from Products.PloneArticle.proxy.fileinnercontent import FileInnerContentProxy
        return utils.getAllAvailableReferenceableTypes(self, FileInnerContentProxy)


    security.declarePublic('listModels')
    def listModels(self,):
        """
        Return a sorted list of all registered models
        """
        registry = getModelRegistry()
        models = registry.values()
        models.sort()
        return models

    security.declarePublic('getReferenceablePortalTypesFor')
    def getReferenceablePortalTypesFor(self, proxy, field):
        """
        Return a list of portal types that this InnerProxy instance can
        reference.
        """
        assert(isinstance(proxy, BaseInnerContentProxy))
        assert(isinstance(field, BaseInnerContentField))

        article = proxy.getArticleObject()
        article_type = article.portal_type
        ref_types = self.referenceable_types

        if not ref_types.has_key(article_type):
            self.updateReferenceableTypes()
        proxy_dict = ref_types[article_type]
        return proxy_dict[field.getName()]

    security.declareProtected(CCP.ManagePortal, 'updateReferenceableTypes')
    def updateReferenceableTypes(self, updates={}, reset=False):
        """
        Update InnerProxy referenceables types structure. This structure is a dict:

        {'PloneArticle': {
                'files': ['File'],
                'images': ['Image', 'News Item'],
                'links': ['Favorite', 'Link']
                },
         'ContentUsingInnerFields': {...},
        }

        Without any param, this method will update this mapping, detecting new
        types/fields and preserving existing entries.

        @param updates: a dict like referenceable_types. Its entries will
        override default values (i.e constrain allowed types). Unsupported types
        for reference by the proxy will be ignored.

        @param reset: reset all mapping, allowing to reference all types
        supported by InnerProxies. If reset, the param 'updates' is ignored.
        """

        at = getToolByName(self, 'archetype_tool')
        ref_types_info = self.referenceable_types
        registered_types = at.listRegisteredTypes()
        registered_portal_types = [i['portal_type'] for i in registered_types]
        pa_types = at.listPortalTypesWithInterfaces([IPloneArticle,])
        proxy_types = at.listPortalTypesWithInterfaces([IBaseInnerContentProxy,])
        proxy_types = [fti.getId() for fti in proxy_types]

        for fti in pa_types:
            portal_type = fti.getId()
            idx = registered_portal_types.index(portal_type) #ValueError no idx
            ## this is the schema of the PA like object
            schema = registered_types[idx]['schema']

            if portal_type not in ref_types_info:
                ## this is the case when a new PA like type has just been installed
                ref_types_info[portal_type] = {}
            pti = ref_types_info[portal_type]

            pt_update = updates.get(portal_type, {})

            for field in schema.fields():
                field_name = field.getName()
                field_update = pt_update.get(field_name)

                ## I the two next tests, we will skip if field is not
                ## updated or reset
                if not isinstance(field, BaseInnerContentField):
                    ## this field is just a simple field, not a proxy, so no update
                    continue

                if (
                    not field_update ## we have no reference in the update table
                    and pti.has_key(field_name) ## a field already exist
                    and not reset ## we don't want to reset
                ):
                    continue

                ## we look for interfaces of inner types
                inner_pt = field.inner_portal_type
                inner_class_idx = registered_portal_types.index(inner_pt)
                inner_class = registered_types[inner_class_idx]['klass']

                ct = utils.getAllAvailableReferenceableTypes(self, inner_class)

                ## here we keep the old ones if reset is not wanted
                if field_update and not reset:
                    new_ct = [pt for pt in ct if pt in field_update]
                    ct = new_ct

                pti[field_name] = ct
        return

    security.declarePublic('getThumbnailTag')
    def getThumbnailTag(self, instance, field_name, **kwargs):
        """Generate an html img tag like OFS.Image but use specific thumb url"""

        # Get field
        field = instance.getField(field_name)

        # Get image
        accessor = field.getAccessor(instance)
        img = accessor()

        # If img doesn't exist returns an empty tag
        if not img:
            return ''

        # Check image class
        if not isinstance(img, Image):
            raise ValueError, 'Scale can only be applied on Image'

        # Get scale size
        width = kwargs.get('width', img.width)
        height = kwargs.get('height', img.height)
        maximizeTo = kwargs.get('maximizeTo', 0)
        maxRatio = kwargs.get('maxRatio', 0)
        width, height = self._getScaleSize(img, width, height, maximizeTo, maxRatio)

        # Get resource information
        content_type = img.content_type
        mtool = getToolByName(self, 'mimetypes_registry')
        mimes = mtool.lookup(content_type)
        ext = str(mimes[0].extensions[0])

        # Get some attributes
        title = kwargs.get('title', instance.title_or_id())
        alt = kwargs.get('alt', title)

        # Get tag url
        tag_url = instance.absolute_url()

        if HAS_PIL:
            tag_url += '/pa_thumb/%(field_name)sx%(width)sx%(height)s.%(ext)s' % {
                'field_name': field_name,
                'width': width,
                'height': height,
                'ext': ext,
                }

        values = {'src' : tag_url,
                  'alt' : escape(alt, 1),
                  'title' : escape(title, 1),
                  'height' : height,
                  'width' : width,
                 }

        result = '<img src="%(src)s" alt="%(alt)s" title="%(title)s" '\
                 'height="%(height)s" width="%(width)s"' % values

        for key, value in kwargs.items():
            if key not in ('width', 'height', 'title', 'alt',) and value:
                result = '%s %s="%s"' % (result, key, value)

        return '%s />' % result

    security.declarePublic('getThumbnail')
    def getThumbnail(self, instance, thumbnail_id, request=None):
        """Create scale of image

        @param instance: Content with ImageField
        @param thumbnail_id: <field_name>x<width>x<height>.<ext>"""

        if request is None:
            request = self.REQUEST
        response = request.RESPONSE

        # Parse thumbnail_id
        match = THUMBNAIL_RE.match(thumbnail_id)
        if match is None:
            raise ValueError, "Thumbnail id is bad formated"

        field_name = match.group('field_name')
        width = int(match.group('width'))
        height = int(match.group('height'))

        # Get image and scale it
        field = instance.getField(field_name)
        accessor = field.getAccessor(instance)
        img = accessor()

        return self.scaleImage(img, width, height).index_html(request, response)

    security.declarePublic('scaleImage')
    def scaleImage(self, img, width, height):
        """Return a scaled OFS.Image instance

        @param img: an OFS.Image instance.
        @param width: width in pixels
        @param height: height in pixels

        if img is not an Image, it is returned as is.
        """

        if type(img) in (StringType,) and len(img) == 0:
            return ''

        # Check image
        if not isinstance(img, Image):
            raise ValueError, 'Scale can only be applied on Image'

        if not HAS_PIL:
            return img

        # empty string - stop rescaling because PIL fails on an empty string
        if img.get_size() == 0:
            return ''

        # If null size returns empty string
        if width == 0 or height == 0:
            return ''

        size = int(width), int(height)
        image = PIL.Image.open(StringIO(img.data))
        # consider image mode when scaling
        # source images can be mode '1','L,','P','RGB(A)'
        # convert to greyscale or RGBA before scaling
        # preserve palletted mode (but not pallette)
        # for palletted-only image formats, e.g. GIF
        # PNG compression is OK for RGBA thumbnails
        original_mode = image.mode
        if original_mode == '1':
            image = image.convert('L')
        elif original_mode == 'P':
            image = image.convert('RGBA')
        image.thumbnail(size, PIL.Image.ANTIALIAS)
        # XXX: tweak to make the unit test
        #      test_fields.ProcessingTest.test_processing_fieldset run
        format = image.format and image.format or 'GIF'
        # decided to only preserve palletted mode
        # for GIF, could also use image.format in ('GIF','PNG')
        if original_mode == 'P' and format == 'GIF':
            image = image.convert('P')
        thumbnail_file = StringIO()
        # quality parameter doesn't affect lossless formats
        image.save(thumbnail_file, format, quality=88)
        thumbnail_file.seek(0)
        return Image('thumb', 'thumb', thumbnail_file, content_type='image/png')

    def _getScaleSize(self, img, width, height, maximizeTo=0, maxRatio=0):
        """Returns tuple(width, height) of scale
           if maximizeTo!=0 can return bigger dimensions than original
           if maxRatio=1 ratio is based on biggest width or height"""

        img_width = img.width
        img_height = img.height

        if img_width=="" or img_height=="":
            # img width and height not initiated
            return (0,0)

        width_ratio = 1.0
        height_ratio = 1.0
        if maximizeTo  :
           # ratio independant from original width, height
           width_ratio = float(maximizeTo)/img_width
           height_ratio = float(maximizeTo)/img_height
        else:
            # Get standard ratio
            if width is not None and width < img_width:
                width_ratio = float(width)/img_width
            if height is not None and height < img_height:
                height_ratio = float(height)/img_height
        if maxRatio :
            ratio = max(width_ratio, height_ratio)
        else:
            ratio = min(width_ratio, height_ratio)

        # Returned the scale size
        return int(ratio*img_width), int(ratio*img_height)

    security.declareProtected(CCP.ManagePortal, 'migrate')
    def migrate(self, **options):
        """
            Migrate articles
            See migrator documentation for options.
        """
        from Products.PloneArticle.migration import migrator
        return migrator.migrate(self, **options)

    security.declarePublic('cleanFilename')
    def cleanFilename(self, filename=''):
        """
        IE uploads file with full paths. This method return the single filename.
        """
        assert isinstance(filename, basestring)
        filename = os.path.basename(filename)
        return filename.split("\\")[-1]

    security.declarePublic('fileFromUpload')
    def fileFromUpload(self, id, title, uploaded_file, article):
        #f = File(id, title, uploaded_file)
        f = BaseUnit(id, uploaded_file, article)
        f.title = title # OFS File property
        return f

    security.declarePublic('getContentTypeOf')
    def getContentTypeOf(self, file_object, context):
        """
        return getContentType of file

        @param file_object: a File instance (or BaseUnit)

        @param context: the acquisition context to set on file. Required for
        security checks
        """
        return file_object.__of__(context).getContentType()

    security.declarePublic('imageFromUpload')
    def imageFromUpload(self, id, title, image_file):
        image = Image(id, title, image_file)
        return image

    security.declarePublic('getImageDimensions')
    def getImageDimensions(self, image):
        """
        Return width/height from an Image instance.
        The width and height are guaranteed to be integers.
        In case width and height can't be recovered, something
        must have gone wrong, and 0 is returned.
        This method is helpful for unprivileged scripts
        """
        width=0
        height=0
        # Try fetching width and height as int. If it fails, fall-back to 0.
        try:
            width=int(image.width)
        except:
            pass
        try:
            height=int(image.height)
        except:
            pass
        return width, height

    security.declarePublic('useAttachmentField')
    def useAttachmentField(self):
        """Returns True if AttachmentField product is installed"""

        return USE_ATTACHMENT_FIELD

    ### Version management

    security.declareProtected(CCP.ManagePortal, 'getVersion')
    def getVersion(self):
        """Get internal numversion and version
        """
        return self._numversion, self._version

    security.declareProtected(CCP.ManagePortal, 'getVersionFromFS')
    def getVersionFromFS(self):
        """Get numversion and version from FS
        """
        from Products.PloneArticle import __pkginfo__ as pkginfo
        return pkginfo.numversion, pkginfo.version.lower()

    security.declareProtected(CCP.View, 'needsVersionMigration')
    def needsVersionMigration(self):
        """Version migration is required when fs version != installed version
        """
        nv, v = self.getVersion()
        fsnv, fsv = self.getVersionFromFS()
        return nv != fsnv # or v != fsv ## fail for CVS/SVN

    security.declareProtected(CCP.ManagePortal, 'setInstanceVersion')
    def setInstanceVersion(self, version):
        """ The version this instance of plonearticle is on """
        self._version = version
        major, minor, suffix =  version.split('.')
        suffix = suffix.split('-')
        bugfix = suffix[0]
        release = len(suffix) > 1 and suffix[1] or ''
        self._numversion = (int(major), int(minor), int(bugfix), release)

    security.declareProtected(CCP.ManagePortal, 'setVersionFromFS')
    def setVersionFromFS(self):
        """Updates internal numversion and version from FS
        """
        self._numversion, self._version = self.getVersionFromFS()


    # add an exclamation in ZMI if migration needed
    def om_icons(self):
        icons = ({
                    'path':'misc_/PloneArticle/tool.gif',
                    'alt':self.meta_type,
                    'title':self.meta_type,
                 },)
        if self.needsVersionMigration():
            icons = icons + ({
                     'path':'misc_/PageTemplates/exclamation.gif',
                     'alt':'Error',
                     'title':'PloneArticle needs updating'
                  },)

        return icons

    # migration stuff
    security.declareProtected(CCP.ManagePortal, 'knownVersions')
    def knownVersions(self):
        """ All known version ids, except current one """
        versions = _upgradePaths.keys()
        versions.sort()
        return versions

    def _upgrade(self, version):
        version = version.lower()
        if not _upgradePaths.has_key(version):
            return None, ("Migration completed at version %s" % version,)

        newversion, function = _upgradePaths[version]
        res = function(self.aq_parent)
        return newversion, res

    security.declareProtected(CCP.ManagePortal, 'upgrade')
    def upgrade(self, REQUEST=None, dry_run=None, swallow_errors=True,
                force_instance_version=None):

        result_log = []
        instance_version = force_instance_version or self.getVersion()[1]

        if dry_run:
            result_log.append(("Dry run selected.", logging.INFO))

        # either get the forced upgrade instance or the current instance
        newv = getattr(REQUEST, "force_instance_version", instance_version)

        result_log.append(("Starting the migration from "
                    "version: %s" % newv, logging.INFO))

        while newv is not None:
            result_log.append(
                ("Attempting to upgrade from: %s" % newv, logging.INFO))
            try:
                newv, msgs = self._upgrade(newv)
                if msgs:
                    for msg in msgs:
                        # if string make list
                        if isinstance(msg, basestring):
                            msg = [msg,]
                        # if no status, add one
                        if len(msg) == 1:
                            msg.append(logging.INFO)
                        result_log.append(msg)
                if newv is not None:
                    result_log.append(("Upgrade to: %s, completed" % newv, logging.INFO))
                    self.setInstanceVersion(newv)

            except ConflictError:
                raise
            except:
                exc_type, exc_value, exc_tb = sys.exc_info()
                result_log.append(("Upgrade aborted", logging.ERROR))
                result_log.append(("Error type: %s" % exc_type, logging.ERROR))
                result_log.append(("Error value: %s" % exc_value, logging.ERROR))
                for line in traceback.format_tb(exc_tb):
                    result_log.append((line, logging.ERROR))

                # set newv to None
                # to break the loop
                newv = None
                if not swallow_errors:
                    for msg, sev in result_log: LOG.log(sev, msg)
                    raise
                else:
                    # abort transaction to safe the zodb
                    transaction.abort()

        result_log.append(("End of upgrade path, migration has finished", logging.INFO))

        if self.needsVersionMigration():
            result_log.append((("The upgrade path did NOT reach "
                        "current version"), logging.ERROR))
            result_log.append(("Migration has failed", logging.ERROR))
        else:
            result_log.append((
                "Your PloneArticle instance is now up-to-date.", logging.INFO))

        if dry_run:
            result_log.append(("Dry run selected, transaction aborted",
                               logging.INFO))
            transaction.abort()

        # log all this
        for msg, sev in result_log:
            LOG.log(sev, msg)

        return result_log

    ###
    ## Configlet helpers
    ###

    security.declareProtected(CCP.ManagePortal, 'configletThumbnails')
    def configletThumbnails(self, REQUEST=None):
        """Provides a structure for configlet thumbnails as
        [{'label': xxx, 'url': xxx, 'cssclass': xxx}, ...]
        """

        if REQUEST is None:
            REQUEST = self.REQUEST

        out = []
        tab_defs = (
            # (translated title, template id),
            (ArticleMessageFactory(u'label_article_preferences',
                                   default=u'PloneArticle Preferences'),
             'pa_management_form'),
            (PloneMessageFactory(u'models', default=u'Models')
             , 'pa_manage_models_form'),
            (PloneMessageFactory(u'images', default=u'Images'),
             'pa_manage_images_form'),
            (PloneMessageFactory(u'files', default=u'Attachments'),
             'pa_manage_attachments_form'),
            (PloneMessageFactory(u'links', default=u'Links'),
             'pa_manage_links_form')
            )

        for message, template_id in tab_defs:
            thumbnail = {
                'label': message,
                'url': self.absolute_url() + '/' + template_id,
                'css_class': None
                }
            if REQUEST.URL.endswith(template_id):
                thumbnail['css_class'] = "selected"
            out.append(thumbnail)
        return out


    security.declarePublic('isArticle')
    def isArticle(self, obj):
        """
        Return True if obj is an Article
        Used by portal_javascript/portal_css ...
        """
        if IPloneArticle.providedBy(obj) :
            return True


    security.declarePublic('isEditableArticle')
    def isEditableArticle(self, obj):
        """
        Return True if obj is an Article
        and user has permission to modify it.
        Used by portal_javascript/portal_css ...
        """
        if self.isArticle(obj):
            mtool = getToolByName(self, 'portal_membership')
            if mtool.checkPermission('Modify portal content', obj) :
                return True
                

    security.declarePublic('isEditableArticle')
    def useJquery(self, obj):
        """
        Return True if we're in an article
        and version of plone < 3.1
        """
        pmig = getToolByName(self, 'portal_migration')
        if IPloneArticle.providedBy(obj) and \
           pmig.getFSVersionTuple() < (3, 1) :    
            return True          


InitializeClass(PloneArticleTool)
registerToolInterface(PLONEARTICLE_TOOL, IPloneArticleTool)



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: framework.py 2163 2006-10-27 13:23:55Z roeder $
##############################################################################
#
# ZopeTestCase 
#
# COPY THIS FILE TO YOUR 'tests' DIRECTORY.
#
# This version of framework.py will use the SOFTWARE_HOME
# environment variable to locate Zope and the Testing package.
#
# If the tests are run in an INSTANCE_HOME installation of Zope,
# Products.__path__ and sys.path with be adjusted to include the
# instance's Products and lib/python directories respectively.
#
# If you explicitly set INSTANCE_HOME prior to running the tests,
# auto-detection is disabled and the specified path will be used 
# instead.
#
# If the 'tests' directory contains a custom_zodb.py file, INSTANCE_HOME
# will be adjusted to use it.
#
# If you set the ZEO_INSTANCE_HOME environment variable a ZEO setup 
# is assumed, and you can attach to a running ZEO server (via the 
# instance's custom_zodb.py).
#
##############################################################################
#
# The following code should be at the top of every test module:
#
# import os, sys
# if __name__ == '__main__':
#     execfile(os.path.join(sys.path[0], 'framework.py'))
#
# ...and the following at the bottom:
#
# if __name__ == '__main__':
#     framework()
#
##############################################################################

__version__ = '0.2.3'

# Save start state
#
__SOFTWARE_HOME = os.environ.get('SOFTWARE_HOME', '')
__INSTANCE_HOME = os.environ.get('INSTANCE_HOME', '')

if __SOFTWARE_HOME.endswith(os.sep):
    __SOFTWARE_HOME = os.path.dirname(__SOFTWARE_HOME)

if __INSTANCE_HOME.endswith(os.sep):
    __INSTANCE_HOME = os.path.dirname(__INSTANCE_HOME)

# Find and import the Testing package
#
if not sys.modules.has_key('Testing'):
    p0 = sys.path[0]
    if p0 and __name__ == '__main__':
        os.chdir(p0)
        p0 = ''
    s = __SOFTWARE_HOME
    p = d = s and s or os.getcwd()
    while d:
        if os.path.isdir(os.path.join(p, 'Testing')):
            zope_home = os.path.dirname(os.path.dirname(p))
            sys.path[:1] = [p0, p, zope_home]
            break
        p, d = s and ('','') or os.path.split(p)
    else:
        print 'Unable to locate Testing package.',
        print 'You might need to set SOFTWARE_HOME.'
        sys.exit(1)

import Testing, unittest
execfile(os.path.join(os.path.dirname(Testing.__file__), 'common.py'))

# Include ZopeTestCase support
#
if 1:   # Create a new scope

    p = os.path.join(os.path.dirname(Testing.__file__), 'ZopeTestCase')

    if not os.path.isdir(p):
        print 'Unable to locate ZopeTestCase package.',
        print 'You might need to install ZopeTestCase.'
        sys.exit(1)

    ztc_common = 'ztc_common.py'
    ztc_common_global = os.path.join(p, ztc_common)

    f = 0
    if os.path.exists(ztc_common_global):
        execfile(ztc_common_global)
        f = 1
    if os.path.exists(ztc_common):
        execfile(ztc_common)
        f = 1

    if not f:
        print 'Unable to locate %s.' % ztc_common
        sys.exit(1)

# Debug
#
print 'SOFTWARE_HOME: %s' % os.environ.get('SOFTWARE_HOME', 'Not set')
print 'INSTANCE_HOME: %s' % os.environ.get('INSTANCE_HOME', 'Not set')
sys.stdout.flush()




## Script (Python) "getBrowserCrumbs"
##title=Helper method for getting browser crumbs
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=obj=None

if not obj : 
   obj=context

from Products.CMFCore.utils import getToolByName

ct = getToolByName(context, 'portal_catalog')
stp = getToolByName(context, 'portal_properties').site_properties
query = {}


currentPath = '/'.join(obj.getPhysicalPath())
query['path'] = {'query':currentPath, 'navtree':1, 'depth': 0}

rawresult = ct(**query)

# Sort items on path length
dec_result = [(len(r.getPath()),r) for r in rawresult]
dec_result.sort(lambda x, y: cmp(x[0], y[0]))

result = []
for r_tuple in dec_result:
    item = r_tuple[1]
    item_path = item.getPath()
    data = {'Title': item.pretty_title_or_id(),
            'path': item_path}
    result.append(data)
return result
